//----------------------------------------------------------------------
//	Copyright  1990-1998 by Sienna Software, Inc., All rights reserved.
//
//	File:		Controller.cpp
//
//	Contains:	Skelton code for a Starry Night SETi at home plugin. This is a simple version that 
//				could easily be expanded, to for instance, support the various history 
//				programs that keep track of all of the units that have been done. This plugin
//				only tracks the current unit.
//
//
//				Starry Night communicates with your plug-in through the DoOperation
//				function (see description below). New with version 3 of Starry Night
//				is the ability for the plugin you write to call Starry Night.
//	
//	Authors:	Ted Leckie, Tom Andersen
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Version 3 offers callbacks, also x-platform plug-ins
//	99/02/20	Tom			3.1			SETI plugin
//
//----------------------------------------------------------------------

// we have to figure out whether we are on macintosh power pc or windows.
// This bit of muck will need to be changed if you use compilers other than those by 
// Metrowerks (www.metrowerks.com) or Microsoft.
// If you are only building for Windows or only for Mac you can just set these
// For instance with a Windows only plug-in define TARGET_OS_WIN32 as true and
// TARGET_OS_MAC as false. (StarryNightPlugins.h uses these defines since it can be used
// on both Macintosh and Windows).
#if defined(_M_MPPC)	/* Visual C++ with Macintosh PowerPC target */
	#define TARGET_OS_MAC 1
	#define TARGET_OS_WIN32 0
#elif defined(_M_IX86)	/* Visual C++ with Intel x86 target */
	#define TARGET_OS_MAC 0
	#define TARGET_OS_WIN32 1
#elif defined(macintosh) /* Metrowerks for Macintosh */
	#define TARGET_OS_MAC 1
	#define TARGET_OS_WIN32 0
#elif defined(__INTEL__) /* Metrowerks for Intel*/
	#define TARGET_OS_MAC 0
	#define TARGET_OS_WIN32 1
#else
	you must define TARGET_OS_MAC and TARGET_OS_WIN32
#endif

// now add any header files we need to
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <ctype.h>

#include "StarryNightPlugins.h"

// On the mac, I added system headers via a pre-compiled header option. 
#if TARGET_OS_WIN32
#include <windows.h>
HINSTANCE gDLLInstance;
// Sorry about this. This is how you get the DC that is being used to draw the screen.
extern pascal HDC GetPortHDC(pGWorldPtr port);
HDC gCurrentDC = NULL;
#endif

// when we are in debug mode, we do extra checks to see if everything is working.
// for the shipping plugin you  define this as 0
#define debug_mode 1



#define noError		0

const unsigned long	Command_Slew		= 20000;
const unsigned long	Command_Synch		= 20001;

// global variables
//-------------------------
PCentreAction	gCentreAction;
long gClientAPIVersion = kVersion1Plugin;		// we don't know what the client will support until op_Initialize is called

// We also keep a flag that tells us whether we are in tracking mode or not.
pBoolean gIsTracking = false;


#if TARGET_OS_MAC
// Starry Night will open the resource fork of this plugin if there is one.
// If you are using resources, you can make sure you have the right resource fork open by 
// setting this up
long gResourceFork = -1;	 
#endif

// This colour structure is used to pass around colours inside the plugin. 
// white would be red = 1.0, green = 1.0, blue = 1.0, black is all 0's
struct TRGBColor {
	float	red;		// 0.0 means no red, 1.0 means full red
	float	green;		// 0.0 means no green, 1.0 means full green
	float	blue;		// 0.0 means no blue, 1.0 means full blue
};
typedef struct TRGBColor TRGBColor;

// function prototypes
//----------------------

// utilities
double ClockSec(void);
void NoteToUser(long inResourceID);
unsigned char *PStringAssign(unsigned char *destination, const char *source);
double ExtractDoubleAfterString(const char* inBigString, const char* inStringToLookFor, pBoolean& worked);
snErr ExtractThreeDoublesAfterString(const char* inBigString, const char* inStringToLookFor, double& out1, double& out2, double& out3);
void HandleError(long inSNErr);
void CallBackErrorHandler(long inSNErr);
snErr RaDecToXYZ(double inRaDegrees, double inDecDegrees, PXYZCoordinates& outXYZ);
void OSMoveTo(double inH, double inV);
void OSLineTo(double inH, double inV, TRGBColor inColor);
void OSDrawText(double inH, double inV, char* inTextString, const TRGBColor& inColor, pBoolean inUnderline = false);
snErr RaDecLineTo(sn_WindowRef inWindowRef, double inStartRADeg, double inStartDecDeg, double inEndRADeg, double inEndDecDeg, TRGBColor inColor);
snErr XYZLineTo(sn_WindowRef inWindowRef, PXYZCoordinates& inFrom, PXYZCoordinates& inTo, TRGBColor inColor);
void AddAndNormalizeVectors(const PXYZCoordinates& inV1, const PXYZCoordinates& inV2, double inFactorFor2, PXYZCoordinates& outResult);
void DoCrossProduct(const PXYZCoordinates& inV1, const PXYZCoordinates& inV2, PXYZCoordinates& outResult);
void NormalizeVector(PXYZCoordinates& inV);
pBoolean DrawGraph(sn_WindowRef inWindowRef, double inH, double inV, double inGraphWidth, double inGraphHeight, double* inPotentials, long inNumPotentials);
pBoolean PointInRect(pPoint inPoint, pRect inRect);
#define OurMax(a,b)            (((a) > (b)) ? (a) : (b))
#define OurMin(a,b)            (((a) < (b)) ? (a) : (b))


#if TARGET_OS_WIN32
unsigned long TRGBToColorRef(const TRGBColor& inColour);
void ColorRefToTRGB(unsigned long inColorRef, TRGBColor& outColour);
#endif

// plugin calls 
void DoInitialize(PInitialize* outInitRecord);
// The protype for DoOperation is supplied by Sienna in Starry Night Plugins.h



// Error string numbers 
const long kSeemsThereIsAProblem = 500;

//http://setiathome.ssl.berkeley.edu/sciencepaper.html
// says that the width is 0.1 degrees.
const double kSETIBeamWidthDeg = 0.1;
const long kTextSize = 12;
const long kLineHeight = kTextSize + 2;
const long kGraphWidth = 250;
const long kGraphHeight = kGraphWidth/4.0;


// some math constants
const double kPiDefine	= 3.14159265358979323846;	// pi base unit used to calculate others
const double kDegreesToRadians = kPiDefine/180.0;
const double kRadiansToDegrees = 1.0/kDegreesToRadians;
const double kDegreesPerHour = 15.0;
const double kAuToMeters = 1.4959787066e11;			// number of metres per astronomical unit
const double kMetersToAu = 1.0/kAuToMeters;			// number of astronomical unit per metres
const double kLightYearsPerParsec	=3.261633;					// Pete got this number from somewhere
const double kParsecsPerAu 			= 4.848136811e-6;			// explanatory suppl, page 716
const double kAuPerParsec 			= 1.0/kParsecsPerAu;		// explanatory suppl, page 716
const double kAuPerLightYear 		= kAuPerParsec/kLightYearsPerParsec;	
const double kLightYearPerAu 		= 1.0/kAuPerLightYear;		
const double kMetrePerLightYear		=kAuPerLightYear*kMetersToAu;


// storage:
// store stuff in static arrays to make like simpler.
//---------------------------------------------------
const long kReadBufferSize = 32000;
char gReadBuffer[kReadBufferSize];
pBoolean gSlewToSETIAtNextChance = false; 
pRect gButtonRect;
sn_WindowRef gLastDrawnWindow;
char kURLString[] = "http://www.livesky.com/seti";
//----------------------------------------------------------------------
//	Data storage for SETI results
//
//	Purpose:	These are the structures that hold the SETI results.
//				use floats in these structures to save space.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------

// This is the status of the plugin. When we first start up, we don't know our status.
enum SETIStatus {	SETIStatus_Unknown = 0,				
					SETIStatus_NoDataFound,			
					SETIStatus_Good			
				};

const long kMaxPositions = 100;
// All angles are stored as radians, as that is what all of the routines use.
struct TWorkUnit {
	SETIStatus 	status;
	double		start_ra;
	double		start_dec;
	double		end_ra;
	double		end_dec;
	double		angle_range;
	double		time_recorded;
	double		subband_center;
	double		subband_base;
	double		subband_sample_rate;
	long		num_positions;
	double		time[kMaxPositions];
	double		ra[kMaxPositions];
	double		dec[kMaxPositions];
};
typedef struct TWorkUnit TWorkUnit;

TWorkUnit gWorkUnit;


const kMaxPotentials = 129;
struct TWorkState {
	long	 	ncfft;			// num ffts. Always from 0 to 9024?
	double 		cr;				// ??
	double 		fl;				// ??
	double		cpu;			// cpu time in seconds
	double		prog;			// ??
	double		bs_power;		// best spike power
	double		bs_score;		// best spike score
	double		bs_bin;			// best spike bin?
	double		bs_fft_ind;		// best spike fft ind?
	double		bs_chirp_rate;	// chirp rate
	long		bs_fft_len;				// fft length
	double		bg_score;				// Best gaussian score
	double		bg_sigma;				// bg_sigma
	double		bg_power;				// bg_power 
	double		bg_true_mean;			// true_mean ?? 
	double		bg_chisq;				// bg_sigma
	long		bg_bin;					// best gaussian bin. What is the bin number?
	double		bg_fft_ind;				// bg_fft_ind ?
	double		bg_chirp_rate;			// bg_chirp_rate
	long		bg_fft_len;				// bg_fft_length 
	long		num_potentials;			// number of potentials
	double		bg_pot[kMaxPotentials];	// The list of potentials
};
typedef struct TWorkState TWorkState;

TWorkState gWorkState;



struct TSpike {
	float	power;
	float	ra;
	float	dec;
	double	time; 		// needs to be a double
	double	freq; 		// needs to be a double
	long	fft_len; 	// fft length - never over 2^31?
	float 	chirp_rate;
};
typedef struct TSpike TSpike;

const long kMaxNumSpikes = 500; 
TSpike	gSpikes[kMaxNumSpikes]; // at 36 bytes each, this is about 16k of RAM
long 	gNumSpikes = 0; 		// this counts the number of spikes that we have stored


// SETI functions
void SetUpSETIResultStorage(void);					// sets up storage for all of the seti data we are to plot
void LoadSETIInfoFromDisk();						// loads in the SETI data from the local disk, filling in the data 
void SlewWindowToSETILocation(sn_WindowRef inWindowRef);	// centres the window on the region where the person is looking
void DrawSETIInWindow(sn_WindowRef inWindowRef);	// Draws the area that SETI is searching in.

// secondary SETI functions
long ReadSETIFile(char* inFileName, char* inBuffer, long inBufferSize);
snErr ParseBufferForWorkUnitInfo(char* inBuffer, long inDataLength);
snErr ParseBufferForSpikes(char* inBuffer, long inDataLength);
snErr ParseBufferForStateInfo(char* inBuffer, long inDataLength);
pBoolean ChanceForSETIAreaToDraw(sn_WindowRef inWindowRef);
double GetPixelSizeOfSETIArea(sn_WindowRef inWindowRef);
pBoolean DrawSETIOutline(sn_WindowRef inWindowRef, double& outMaxH, double& outMidV);
pBoolean DrawSETISpikes(sn_WindowRef inWindowRef);
void GetPerpAndParallelVectors(sn_WindowRef inWindowRef, PXYZCoordinates& outPerpXYZVector, PXYZCoordinates& outParallelXYZVector);
void SortSpikes(TSpike* inSpikes, long inNumberOfSpikes);
void GetSpikeColor(TSpike& inSpike, TRGBColor& outColor);
double GetSpikeDrawSize(TSpike& inSpike);
void GetWindowPoints(sn_WindowRef inWindowRef, double inRa, double inDec, const PXYZCoordinates& inPerpXYZ, double& outTopH, double& outTopV, double& outBottomH, double& outBottomV);
void DrawSETIText(sn_WindowRef inWindowRef, double inH, double inV);
 


//----------------------------------------------------------------------
//	ClockSec
//
//	Purpose:	returns time since computer started or some other arbitrary start, in seconds 
//					
//	Inputs:		ResourceID		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Cross platform telescope controller
//
//----------------------------------------------------------------------

double ClockSec(void)
{
	double rawTime = clock();
	double timeSec = rawTime/CLOCKS_PER_SEC;
	return timeSec;
}

//----------------------------------------------------------------------
//	PointInRect
//
//	Purpose:	returns true if the passed point is in the rect
//					
//	Inputs:		ResourceID		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Cross platform telescope controller
//
//----------------------------------------------------------------------
pBoolean PointInRect(pPoint inPoint, pRect inRect)
{
	if (inPoint.h > inRect.left && inPoint.h < inRect.right && inPoint.v > inRect.top && inPoint.v < inRect.bottom)
		return true;
	else
		return false;
}

//----------------------------------------------------------------------
//	NoteToUser
//
//	Purpose:	sends a message to the user
//					
//	Inputs:		ResourceID		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Cross platform telescope controller
//
//----------------------------------------------------------------------

void NoteToUser(long inResourceID)
{
#if TARGET_OS_MAC
	// interpret the resource as the number of an ALRT resource ID
	long savedResFile = CurResFile();
	UseResFile(gResourceFork);
	OSErr error = NoteAlert(inResourceID, NULL);
	UseResFile(savedResFile);
#endif

#if TARGET_OS_WIN32
	char message[256];
	int worked = LoadString(gDLLInstance, inResourceID, message, 255);
	MessageBox(GetFocus(), message, "Telescope Controller", MB_OK|MB_SYSTEMMODAL);
#endif
}




//----------------------------------------------------------------------
//	PStringAssign
//
//	Purpose:	assigns a c string to a pascal string.
//					
//	Inputs:		source			source string
//				destination		destination string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

unsigned char *PStringAssign(unsigned char *destination, const char *source)
{
	destination[0] = strlen(source);
	for (short charPos = destination[0] + 1; charPos--; destination[charPos+1] = source[charPos])
		;
	return (destination);
}

//----------------------------------------------------------------------
//	DoInitialize
//
//	Purpose:	Starry Night calls our plug-in with op_Initialize at when it is
//				starting up. We inform Starry Night about our needs and wants.
//				We could also perform one time memory allocations, etc here.
//					
//	Inputs:		ioParams	we must fill out this parameter block to inform
//							Starry Night of our needs.
//							If for any reason our initialization fails, we need
//							to set ioParams->fSuccess false.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoInitialize(PInitialize* outInitRecord)
{
#if TARGET_OS_MAC
	gResourceFork = CurResFile();
#endif

	// Starry Night passes us in the outInitRecord->fPluginAPI which version of the
	// plug-in API it supports. We require kVersion5Plugin or later so we check
	// that this is available. If it isn't then we return with fSuccess as false,
	// which causes our plug-in not to be used.
	gClientAPIVersion = outInitRecord->fPluginAPI;
	if (outInitRecord->fPluginAPI < kVersion5Plugin)
	{
		outInitRecord->fSuccess = false;
		return;
	}
	
	outInitRecord->fPluginAPI = kVersion5Plugin;		// this is the API we use
	outInitRecord->fSuccess = true;
	
	// Plug-ins can request that Starry Night place a menu items in its "Settings"
	// menu for it. We will use this menu to implement our main action
	outInitRecord->fWantsMenuItem = true;
	PStringAssign(outInitRecord->fOptionsMenuItem, "Center/Update SETI Position"); // text of menu item,

	// Plug-ins can also request that Starry Night create a floating window that
	// the plug-in can use. We could get fancy and create our own window here,
	// but we don't do that
	outInitRecord->fWantsWindow = false;
	outInitRecord->fWindowMenuItem[0] = 0;

	// If you do any drawing into a Starry Night scene you need to inform
	// Starry Night at which "layer" your drawing will take place. You can
	// think of layers as onion skins in which drawing takes place. So to draw
	// something immediately after constellations are drawn but before the
	// ecliptic guides are drawn then you would set outInitRecord->fLayer to a
	// value between Depth_Constellations and Depth_EclipticGuides.
	// If your plug-in doesn't do any drawing (most don't) then set fLayer
	// as Depth_None.
	// Our SETI plug-in gets Starry Night to draw the areas of SETI searching
	// These outlines are nicest if drawn under the stars and after the guides
	outInitRecord->fLayer = Depth_EclipticGuides + 20;		// pass back Depth_None if you don't do any drawing

	// We pass back that we are NOT a telescope controller. 
	// Starry Night only supports one telescope controller plug-in at a 
	// time and only communicates with the first it encounters. 
	outInitRecord->fIsTelescopeController = false;

	// If you want a menu item for an About Box in Starry Night then set it here.
	// You will be called with an op_About everytime the user selects your menu
	// item.
	outInitRecord->fAboutMenuItem[0] = 0;  // we don't want an about menu item, since we have no window to show for one.

	// If you want to be called periodically by Starry Night everytime there is an
	// idle moment (this can be quite frequently) then you should return fWantsIdleTime
	// as true. For instance you could use this idle time to track a telescope that
	// is slewing from one location to another by drawing a moving field of view
	// indicator. We dont need to be called at idle, since we just look up the 
	// SETI file everytime the menu item is selected.
	outInitRecord->fWantsIdleTime = true;	// if true then plug-in receives op_Idle regularly
	
	gWorkUnit.status = SETIStatus_Unknown;
}



//----------------------------------------------------------------------
//	HandleError
//
//	Purpose:	Called when an actual error occurs
//					
//	Inputs:		inSNErr	description of error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/18	Tom			3.0			New Callback API
//
//----------------------------------------------------------------------

void HandleError(long /*inSNErr*/)
{
	return;
}


//----------------------------------------------------------------------
//	CallBackErrorHandler
//
//	Purpose:	This call is an outline of how you might handle an erorr 
// 				coming back from one of the Sienna API calls
//				In this version, I just have a place to put a break point
//				in with the debugger, but you could do anything.
//					
//	Inputs:		inSNErr	description of possible error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/18	Tom			3.0			New Callback API
//
//----------------------------------------------------------------------

void CallBackErrorHandler(long inSNErr)
{
	if (inSNErr != snNoErr)
	{
		HandleError(inSNErr);
	}
	else
	{
		// everything is fine
	}
}



//----------------------------------------------------------------------
//	DoOperation
//
//	Purpose:	Starry Night communicates with your plug-in through one call, DoOperation.
//				(and hence DoOperation is exported using EXP).
//					
//	Inputs:		inOperation	the operation that Starry Night wishes your plug-in to perform
//							(StarryNightPlugins.h describes each operation). You need not
//							support every operation. This example responds to all the
//							operations that a telescope controller plug-in will need to
//							respond to. You may chose to respond to others as well, if you
//							wish.
//
//	I/O:		ioParams	a parameter block that passes various information back and forth
//							between Starry Night and the plug-in. Each operation uses
//							a different paramater block or none at all, in which case
//							ioParams will be NULL. The description of the various operations
//							(in StarryNightPlugins.h) describes what ioParams means for each
//							one.
//
//	Outputs:	short		Unused. You should always pass back noError here.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

SNCall short DoOperation(OperationT inOperation, void* ioParams)
{
	// When there is no connection to the telescope, we don't respond to many operations
	switch (inOperation)
	{
		case op_Initialize:
			DoInitialize((PInitialize*) ioParams);
			break;
		case op_Terminate:
			// do any cleanup here. You receive op_Terminate when Starry Night quits.
			break;
		case op_MenuCommand:
			{
				// The ioParams should be NULL
				
				LoadSETIInfoFromDisk(); // we load the seti info fresh every time the user asks for the seti info.
				
				// If there was no data to be found, senf the user off to our website, to get the latest on the SETI program
				if (gWorkUnit.status != SETIStatus_Good)
				{
					if (gClientAPIVersion >= kVersion5Plugin) // we require version 5 API, so this should be there
					{
						sn_OpenURL("http://www.siennasoft.com/seti/notinstalled.shtml", false);
					}
				}
				else
				{
					// The plugin API only allows us to move/resize windows etc on an idle call. 
					// so we set a flag here to tell us to centre in on SETI during the 
					// the next call to idle.
					gSlewToSETIAtNextChance = true;
				}
			}
			break;
		case op_DrawLayer:
			{
				// we are called at every update. It would really slow things down
				// to actually check the SETI files for every update of the screen, 
				// so we only draw the SETI stuff.
				
				// The ioParams are really a pointer to a PDraw object:
				PDraw* theDrawContext = (PDraw*) ioParams;
				if (theDrawContext != NULL)
				{
#if TARGET_OS_WIN32	
					// Get the draw context that we will be drawing into
					gCurrentDC = theDrawContext->fDestinationDC;
#endif	
					DrawSETIInWindow(theDrawContext->fDestinationWindowRef);
#if TARGET_OS_WIN32	
					gCurrentDC = NULL; // set it to null, so we don't try and draw later into a possibly bad HDC
#endif	
				}
			}
			break;
		case op_Idle:
			{
				// for this plugin, the only thing we do at idle is check to see if the user has asked us to zoom in
				// on the SETI region or not.
				// Notice that the op_Idle routine here is very very fast if there is nothing to do.
				// this routine can be called hundreds or more times per second, so 
				// be carful with what you do everytime through here.
				//------------------------------
				if (gSlewToSETIAtNextChance)
					SlewWindowToSETILocation(kFrontWindow);
				gSlewToSETIAtNextChance = false;
			}
			break;
		case op_SceneClick:
			{
				PEvent* theEvent = (PEvent*) ioParams;
				// If the passed event indicates a click on our URL button,
				// we open the url.
				// See if the click was in our window
				if (theEvent != NULL && theEvent->fInWindow == gLastDrawnWindow)
				{
					// See if the click was in the right place
					//------------------------------------
					if (PointInRect(theEvent->fEventRecord->where, gButtonRect))
						sn_OpenURL(kURLString, false);
				}
				
			}
			break;
	}
	return noError;
}



//----------------------------------------------------------------------
//	SetUpSETIResultStorage
//
//	Purpose:	We set up storage to hold the results of the SETI data that 
//				we read from disk.
//					
//	Inputs:		None. In this version, I make it real simple, in that 
//				we store all of the SETI information in a static block.
//				This means no worries about memory leaks. 
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------

void SetUpSETIResultStorage(void)
{
	// sets up storage for all of the seti data we are to plot
	// The main thing that we are interested in is the area of the sky that we are plotting.
	// In SETI 2.0 the file name that contains the area that we are searching is called 
	// work_unit.sah
	// start_ra= 5.147
	// start_dec= 23.26
	// end_ra= 5.176
	// end_dec= 23.26
	// angle_range= 0.403
	
	// Since this simple verison of the plugin only supports reading the current 
	// data point, we only have one workunit.
	
	// Power spikes:
	// The file outfile.sah lists all of the spikes found in the file.
	// We read in a list of spikes from the file, and show them as well on the screen.
	//--------------------------
	// spike: power=1.917676e+02 ra= 5.149 dec= 23.26 time= 2451497.75031 freq=1420507997.96 fft_len=131072 chirp_rate=3.697039e-03
	gNumSpikes = 0; // since we use static storage, this is the only step.
	
	// There are also gaussians. I need the format for those....
	//------------------------------------------------------
}

//----------------------------------------------------------------------
//	LoadSETIInfoFromDisk
//
//	Purpose:	We go to the hard drive on this local computer to find the 
//				results of the current work unit.
//					
//	Inputs:		Read the relevant files into RAM for quick scanning and cross-platform ease. 
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
void LoadSETIInfoFromDisk(void) // loads in the SETI data from the local disk, filling in the data
{
	// since we are reloading, we reset the data
	SetUpSETIResultStorage();
	// now we find and open the files we need.
	// Do this by opening the file, reading what we need into a block of memory, 
	// then using standard 'c' routines to read in the data we need from the block.

	// Read in work unit
	long bytesRead = ReadSETIFile("work_unit.sah", gReadBuffer, kReadBufferSize);   // this has Mac/Win implemtations for finding SETI files
	snErr error1 = ParseBufferForWorkUnitInfo(gReadBuffer, bytesRead); // looks through buffer for work unit info.
	
	// Read in spikes - gaussians
	snErr error2 = 0; 		// error2 is for debugging only. Don't care if there are no spikes
	bytesRead = ReadSETIFile("outfile.sah", gReadBuffer, kReadBufferSize);
	if (bytesRead > 0)  // sometimes there is no outfile info loaded in early in a seti search
		error2 = ParseBufferForSpikes(gReadBuffer, bytesRead); // looks through buffer for spikes.

	// Read in state info
	bytesRead = ReadSETIFile("state.sah", gReadBuffer, kReadBufferSize);   // this has Mac/Win implemtations for finding SETI files
	snErr error3 = ParseBufferForStateInfo(gReadBuffer, bytesRead); // looks through buffer for spikes.
	
	// if everything worked we pass, else fail
	if (error1 == snNoErr && error2 == snNoErr && error3 == snNoErr)
		gWorkUnit.status = SETIStatus_Good;
	else
		gWorkUnit.status = SETIStatus_NoDataFound;
}					 

//----------------------------------------------------------------------
//	ParseBufferForStateInfo
//
//	Purpose:	Given a buffer that contains the contents of the file, state.sah,
//				will parse it to determine the stuff we need for the work unit.
//					
//	Inputs:		inBuffer: A buffer that is null terminated. 
//				inDataLength: the length of data in the buffer. (just so ya know)
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
snErr ParseBufferForStateInfo(char* inBuffer, long inDataLength)
{
	if (inDataLength <= 1)
		return snErr_Failed;
	// we use standard c string calls here.
	// note that the buffer is assumed to be a c string.
	long stringLength = strlen(inBuffer);
	if (stringLength != inDataLength)
	{
		// we have null characters in the SETI data files. 
		// what to do?
		return snErr_Failed;
	}
	pBoolean worked;
	
	TWorkState* theWorkState = &gWorkState; //need this for my debugger to work oK
	
	theWorkState->ncfft = ExtractDoubleAfterString(inBuffer, "ncfft=", worked);
	theWorkState->cr = ExtractDoubleAfterString(inBuffer, "cr=", worked);
	theWorkState->fl = ExtractDoubleAfterString(inBuffer, "fl=", worked);
	theWorkState->cpu = ExtractDoubleAfterString(inBuffer, "cpu=", worked);
	theWorkState->prog = ExtractDoubleAfterString(inBuffer, "prog=", worked);
	
	theWorkState->bs_power = ExtractDoubleAfterString(inBuffer, "bs_power=", worked);
	theWorkState->bs_score = ExtractDoubleAfterString(inBuffer, "bs_score=", worked);
	theWorkState->bs_bin = ExtractDoubleAfterString(inBuffer, "bs_bin=", worked);
	theWorkState->bs_fft_ind = ExtractDoubleAfterString(inBuffer, "bs_fft_ind=", worked);
	theWorkState->bs_chirp_rate = ExtractDoubleAfterString(inBuffer, "bs_chirp_rate=", worked);
	theWorkState->bs_fft_len = ExtractDoubleAfterString(inBuffer, "bs_fft_len=", worked);
	
	theWorkState->bg_score = ExtractDoubleAfterString(inBuffer, "bg_score=", worked);
	theWorkState->bg_sigma = ExtractDoubleAfterString(inBuffer, "bg_sigma=", worked);
	theWorkState->bg_power = ExtractDoubleAfterString(inBuffer, "bg_power=", worked);
	theWorkState->bg_true_mean = ExtractDoubleAfterString(inBuffer, "bg_true_mean=", worked);
	theWorkState->bg_chisq = ExtractDoubleAfterString(inBuffer, "bg_chisq=", worked);
	theWorkState->bg_bin = ExtractDoubleAfterString(inBuffer, "bg_bin=", worked);
	theWorkState->bg_fft_ind = ExtractDoubleAfterString(inBuffer, "bg_fft_ind=", worked);

	// we also need the bg_pot 0, bg_pot 1, etc potentials that this file is tracking.
	// we use these to draw a nice graph.
	// -------------------------------------------------------------------
	theWorkState->num_potentials = 0;
	for (long count = 0; count < kMaxPotentials; count++)
	{
		char coordString[25] = "bg_pot ";
		char numString[10] = "";
		sprintf(numString, "%d", count);
		strcat(coordString, numString);
		strcat(coordString, "=");
		
		theWorkState->bg_pot[count] = (double) ExtractDoubleAfterString(inBuffer, coordString, worked);
		if (!worked)
		{
			theWorkState->num_potentials = count;
			break;
		}
	}
	
	if (theWorkState->cpu != 0.0 && !isnan(theWorkState->cpu))
		return snNoErr;
	else
		return snErr_Failed;
}					



//----------------------------------------------------------------------
//	ParseBufferForWorkUnitInfo
//
//	Purpose:	Given a buffer that contains the contents of the file, work_unit,
//				will parse it to determine the stuff we need for the work unit.
//					
//	Inputs:		inBuffer: A buffer that is null terminated. 
//				inDataLength: the length of data in the buffer. (just so ya know)
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
snErr ParseBufferForWorkUnitInfo(char* inBuffer, long inDataLength)
{
	if (inDataLength <= 1)
		return snErr_Failed;
	// we use standard c string calls here.
	// note that the buffer is assumed to be a c string.
	long stringLength = strlen(inBuffer);
	if (stringLength != inDataLength)
	{
		// we have null characters in the SETI data files. 
		// what to do?
		return snErr_Failed;
	}
	
	// The work_unit file stars includes the following lines:
	// start_ra= 5.147
	// start_dec= 23.26
	// end_ra= 5.176
	// end_dec= 23.26
	// angle_range= 0.403
	pBoolean worked;
	gWorkUnit.start_ra = kDegreesPerHour*ExtractDoubleAfterString(inBuffer, "start_ra=", worked);
	gWorkUnit.start_dec = ExtractDoubleAfterString(inBuffer, "start_dec=", worked);
	gWorkUnit.end_ra = kDegreesPerHour*ExtractDoubleAfterString(inBuffer, "end_ra=", worked);  // store degrees instead of hours ra
	gWorkUnit.end_dec = ExtractDoubleAfterString(inBuffer, "end_dec=", worked);
	gWorkUnit.angle_range = ExtractDoubleAfterString(inBuffer, "angle_range=", worked);
	gWorkUnit.time_recorded = ExtractDoubleAfterString(inBuffer, "time_recorded=", worked);
	gWorkUnit.subband_center = ExtractDoubleAfterString(inBuffer, "subband_center=", worked);
	gWorkUnit.subband_base = ExtractDoubleAfterString(inBuffer, "subband_base=", worked);
	gWorkUnit.subband_sample_rate = ExtractDoubleAfterString(inBuffer, "subband_sample_rate=", worked);
	
	
	// we also need the coord0, coord1, etc positions that this file is tracking.
	// we use these to draw an accurate picture of the SETI search region.
	// -------------------------------------------------------------------
	gWorkUnit.num_positions = ExtractDoubleAfterString(inBuffer, "num_positions=", worked);
	if (gWorkUnit.num_positions > kMaxPositions)
		gWorkUnit.num_positions = kMaxPositions;
	
	for (long count = 0; count < gWorkUnit.num_positions; count++)
	{
		char coordString[25] = "coord";
		char numString[10] = "";
		sprintf(numString, "%d", count);
		strcat(coordString, numString);
		strcat(coordString, "=");
		
		double time;
		double ra;
		double dec;
		ExtractThreeDoublesAfterString(inBuffer, coordString, time, ra, dec);
		ra *= kDegreesPerHour; // to degrees
		gWorkUnit.time[count] = time;
		gWorkUnit.ra[count] = ra;
		gWorkUnit.dec[count] = dec;
	}
	
	if (gWorkUnit.start_ra != 0.0 && !isnan(gWorkUnit.start_ra))
		return snNoErr;
	else
		return snErr_Failed;
}					

//----------------------------------------------------------------------
//	ParseBufferForSpikes
//
//	Purpose:	Given a buffer that contains the contents of the file, work_unit,
//				will parse it to determine the stuff we need for the work unit.
//					
//	Inputs:		inBuffer: A buffer that is null terminated. 
//				inDataLength: the length of data in the buffer. (just so ya know)
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
snErr ParseBufferForSpikes(char* inBuffer, long inDataLength)
{
	if (inDataLength <= 1)
		return snErr_Failed;
	// we use standard c string calls here.
	// note that the buffer is assumed to be a c string.
	long stringLength = strlen(inBuffer);
	if (stringLength != inDataLength)
	{
		// we have null characters in the SETI data files. 
		// what to do?
		return snErr_Failed;
	}
	
	// The spikes are in the file outfile.sah
	// the spikes are in lines as the following:
	// spike: power=1.917676e+02 ra= 5.149 dec= 23.26 time= 2451497.75031 freq=1420507997.96 fft_len=131072 chirp_rate=3.697039e-03
	// We have to go through the buffer, looking for lines that start with the word 'spike'
	// since these are unix text files, we can look for the text
	// "\nspike:"
	// When we find the word 'spike:', we then read in the spike, and then look for the next spike.
	
	char* currentPointer = inBuffer;
	currentPointer = strstr(currentPointer, "\nspike:");
	while ((currentPointer != NULL) && (gNumSpikes < kMaxNumSpikes))
	{
		TSpike currentSpike; // read into loca copy to better debug
		
		pBoolean worked;
		currentSpike.power = ExtractDoubleAfterString(currentPointer, "power=", worked);
		currentSpike.ra = kDegreesPerHour*ExtractDoubleAfterString(currentPointer, "ra=", worked);
		currentSpike.dec = ExtractDoubleAfterString(currentPointer, "dec=", worked);
		currentSpike.time = ExtractDoubleAfterString(currentPointer, "time=", worked);
		currentSpike.freq = ExtractDoubleAfterString(currentPointer, "freq=", worked);
		currentSpike.fft_len = ExtractDoubleAfterString(currentPointer, "fft_len=", worked);
		currentSpike.chirp_rate = ExtractDoubleAfterString(currentPointer, "chirp_rate=", worked);
		
		gSpikes[gNumSpikes] = currentSpike;
		gNumSpikes++;
		currentPointer++; // increment pointer so that we do not find the same line again
		currentPointer = strstr(currentPointer, "\nspike:"); // look for next spike
	}
	
	
	// now we must sort the spike so that the highest power ones are first in the list.
	//------------------------------------------------------------------------------
	SortSpikes(gSpikes, gNumSpikes);
	
	return snNoErr; // return that it worked even if 
}					

//----------------------------------------------------------------------
//	SortSpikes
//
//	Purpose:	Will sort the passed array of spikes by power.
//					
//	Inputs:		inSpikes: an array of spikes
//	Inputs:		inNumberOfSpikes: how many spikes are in the array 
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/28	Tom			3.0			New
//
//----------------------------------------------------------------------
void SortSpikes(TSpike* inSpikes, long inNumberOfSpikes)
{
	TSpike			exchangeRecord;
	long			i,j,gap;
	
	gap = 1;
	do
	{
		gap = 3*gap + 1;
	}
	while (gap <= inNumberOfSpikes);
	for (gap /= 3; gap > 0; gap /= 3)
	{
		for (i = gap; i < inNumberOfSpikes; i++)
		{
			for (j=i-gap; j >= 0 && (inSpikes[j].power < inSpikes[j+gap].power); j -= gap)
			{
				exchangeRecord = inSpikes[j];
				inSpikes[j] = inSpikes[j+gap];
				inSpikes[j+gap] = exchangeRecord;
			}
		}
	}
}

//----------------------------------------------------------------------
//	ExtractDoubleAfterString
//
//	Purpose:	Given a null terminated string, will look for the number 
// 				after the string, and return it. Does not alter either input string.
//					
//	Inputs:		inFileName: a 'c' string name of the file, without any path information. 
//	Inputs:		inBuffer: 
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
double ExtractDoubleAfterString(const char* inBigString, const char* inStringToLookFor, pBoolean& worked)
{
	worked = false;
	long stringLength = strlen(inStringToLookFor);
	char* pointer = strstr(inBigString, inStringToLookFor);
	
	if (pointer == NULL)
		return 0.0;			// should perhaps do a little more error handling
	
	pointer += stringLength;
	// pointer now points to the start of the numeric string that we need to 
	// extract the number from.
	// we remove white space that may be before the number now
	while (isspace(*pointer))
		pointer++;
	
	double theNumber;
	sscanf(pointer, "%lf", &theNumber);
	
	worked = true;
	return theNumber;
}

//----------------------------------------------------------------------
//	ExtractThreeDoublesAfterString
//
//	Purpose:	Given a null terminated string, will look for three numbers after the key string 
// 				, and return them. Does not alter either input string.
//					
//	Inputs:		inFileName: a 'c' string name of the file, without any path information. 
//	Inputs:		inBuffer: 
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
snErr ExtractThreeDoublesAfterString(const char* inBigString, const char* inStringToLookFor, double& out1, double& out2, double& out3)
{
	out1 = 0.0;
	out2 = 0.0;
	out3 = 0.0;
	
	long stringLength = strlen(inStringToLookFor);
	char* pointer = strstr(inBigString, inStringToLookFor);
	
	if (pointer == NULL)
		return snErr_Failed;
	
	pointer += stringLength;
	// pointer now points to the start of the numeric string that we need to 
	// extract the number from.
	// we remove white space that may be before the number now
	while (isspace(*pointer))
		pointer++;
	
	sscanf(pointer, "%lf %lf %lf", &out1, &out2, &out3);
	
	return snNoErr;
}


//----------------------------------------------------------------------
//	SlewWindowToSETILocation
//
//	Purpose:	Given a passed window, will zoom in and slew the window to the coordinates
//				of the current work unit.
//					
//	Inputs:		The window ref can be set to kFrontWindow if you want to slew the window over. 
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
void SlewWindowToSETILocation(sn_WindowRef inWindowRef)
{
	if (gWorkUnit.status == SETIStatus_Unknown)
		LoadSETIInfoFromDisk();
	
	if (gWorkUnit.status != SETIStatus_Good)
		return;
	
	
	// The coords we have are in ra, dec. Most of the routines that a plugin can call in Starry Night 
	// use 'XYZ' coordinates. These coordinates can be obtained using sn_SphericalToXYZ()
	PXYZCoordinates startXYZ;
	RaDecToXYZ(gWorkUnit.start_ra, gWorkUnit.start_dec, startXYZ);
	PXYZCoordinates endXYZ;
	RaDecToXYZ(gWorkUnit.end_ra, gWorkUnit.end_dec, endXYZ);
	
	// add the two together and normalize for the 'average' vector
	PXYZCoordinates midXYZ;
	AddAndNormalizeVectors(startXYZ, endXYZ, +1.0, midXYZ);
	
	// we need a field of view. I think that about 5 times the angle range should do.
	// we need to set a 

	double fieldOfView = gWorkUnit.angle_range*2.0*kDegreesToRadians; // all angles that we pass to Starry Night must be in radians
	
	// we want to set the zoom state for the window and zoom in onto the the place where the 
	// seti search currently is. The drawing will be done in DrawSETIInWindow below

	// do the slew.
	snErr theError = sn_TurnDaylightOnOff(inWindowRef, false); // turn daylight off
	theError = sn_CentreWindowAtCoords(kFrontWindow, kSNCelestialJ2000System, midXYZ, true, true, fieldOfView, true);
}					


//----------------------------------------------------------------------
//	RaDecToXYZ
//
//	Purpose:	Utility call to turn an ra dec into an XYZ coordinate. 
//					
//	Inputs:		ra, dec should be in degrees. 
//
//				This call should be as fast as possible, since it will be called even for views that do not 
//				have the region of interest in them.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
snErr RaDecToXYZ(double inRaDegrees, double inDecDegrees, PXYZCoordinates& outXYZ)
{
	PSphericalCoordinates raDecCoords;
	raDecCoords.ra = inRaDegrees*kDegreesToRadians;// all angles that we pass to Starry Night must be in radians
	raDecCoords.dec = inDecDegrees*kDegreesToRadians;// all angles that we pass to Starry Night must be in radians
	raDecCoords.radius = 1.0;
	return sn_SphericalToXYZ(raDecCoords, &outXYZ);
}					


//----------------------------------------------------------------------
//	DrawSETIInWindow
//
//	Purpose:	Given a passed window, will draw the seti information in it. This routine is called
//				when the window is being drawn off-screen, so any drawing done in here will not be seen
//				until all the layers are done drawing. 
//					
//	Inputs:		The window ref should always be as passed to the plugin in DoOperation(), with op_DrawLayer 
//
//				This call should be as fast as possible, since it will be called even for views that do not 
//				have the region of interest in them.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
void DrawSETIInWindow(sn_WindowRef inWindowRef)
{
	// Set the button box to zero
	gButtonRect.right = gButtonRect.left = gButtonRect.bottom = gButtonRect.top = 0;
	gLastDrawnWindow = NULL;
	
	// Make sure data is loaded
	if (gWorkUnit.status == SETIStatus_Unknown)
		LoadSETIInfoFromDisk();
	
	if (gWorkUnit.status != SETIStatus_Good)
		return;
	
	
	if (!ChanceForSETIAreaToDraw(inWindowRef))
		return;
	
	// Our rendition of the SETI region is a rectangle in the sky, with lines across it to signify where the 
	// signals have been seen. 
	// We draw the SETI region outline in a red, and draw all of the spikes as lines in the +/- dec direction to the
	// -------------------------------------------------------
	double textH;
	double textV;
	DrawSETIOutline(inWindowRef, textH, textV);
	DrawSETISpikes(inWindowRef);
	//DrawGaussians(inWindowRef);
	
	DrawSETIText(inWindowRef, textH, textV);
	
}					

//----------------------------------------------------------------------
//	DrawSETIText
//
//	Purpose:	Draws some text on the screen to please the user.
//				Will draw more text as one zooms in on the SETI region.
//				
//				
//					
//	Inputs:		The window ref should always be as passed to the plugin in DoOperation(), with op_DrawLayer 
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/27	Tom			3.0			New
//
//----------------------------------------------------------------------
void DrawSETIText(sn_WindowRef inWindowRef, double inH, double inV)
{
	TRGBColor textColor;
	textColor.red = 0.8;
	textColor.green = 0.1;
	textColor.blue = 0.0;
	
	OSDrawText(inH, inV, "SETI", textColor);
	inV += kLineHeight;
	
	// we draw 3 levels of info. The most basic level is just the above 'SETI' title
	// The next level is when zoomed in a bit, and the final level is when fully zoomed in.
	//------------------------------------------------------------------------------------
	double theSETISizeRadians = gWorkUnit.angle_range*kDegreesToRadians;
	double windowFieldRadians;
	snErr error = sn_GetFieldOfView(inWindowRef, &windowFieldRadians);
	
	char outText[256];
	// Medium level of detail
	// ----------------------
	if (theSETISizeRadians*15 > windowFieldRadians)
	{
		// Show perentage done, and the number of hours worked
		double percentageDone = gWorkState.ncfft/9024.0*100.0;
		double cpuHours = gWorkState.cpu/3600.0;
		sprintf(outText, "%.2f%% done, %.2f cpu hours", percentageDone, cpuHours);
		OSDrawText(inH, inV, outText, textColor);
		inV += kLineHeight;
	}	
	
	if (theSETISizeRadians*8 > windowFieldRadians)
	{
		// Show best spike power
		sprintf(outText, "Highest spike power: %.4f", gWorkState.bs_power);
		OSDrawText(inH, inV, outText, textColor);
		inV += kLineHeight;
	
		// Show best spike score
		sprintf(outText, "Best spike score: %.4f", gWorkState.bs_score);
		OSDrawText(inH, inV, outText, textColor);
		inV += kLineHeight;
	
		// Show best gaussian power
		sprintf(outText, "Highest gaussian power: %.4f", gWorkState.bg_power);
		OSDrawText(inH, inV, outText, textColor);
		inV += kLineHeight;
	
		// Show best gaussian score
		sprintf(outText, "Best gaussian score: %.4f", gWorkState.bg_score);
		OSDrawText(inH, inV, outText, textColor);
		inV += kLineHeight;
	}
	
	// Highest level of detail
	if (theSETISizeRadians*4 > windowFieldRadians)
	{
		// draw the graph of the potentials for the best gaussian
		//-------------------------------------------------------
		pBoolean graphDrawn = DrawGraph(inWindowRef, inH, inV, kGraphWidth, kGraphHeight, gWorkState.bg_pot, gWorkState.num_potentials);
		
		if (graphDrawn)
		{
			inV += kLineHeight;
			// Draw this inside the graph at the top
			OSDrawText(inH + 1, inV, "Best power over time", textColor);
			
			// Draw this right below the graph
			inV += kGraphHeight;
		}
		
		// Add a URL line
		char helpString[256] = "More SETI Info...";
		//strcat(helpString, kURLString);
		//strcat(helpString, "for more information");
		textColor.red = 0.0;
		textColor.green = 0.3;
		textColor.blue = 0.99; // Make the internet link blue
		OSDrawText(inH, inV, helpString, textColor, true);
		
		const long kAssumedTextLength = 250;
		gButtonRect.top = inV - kLineHeight;
		gButtonRect.bottom = inV;
		gButtonRect.left = inH;
		gButtonRect.right = inH + kAssumedTextLength; // kind of a hack, but what the hey. Should use StringLength, or like that
		gLastDrawnWindow = inWindowRef;
	}

}
//----------------------------------------------------------------------
//	DrawGraph
//
//	Purpose:	Draws a little graph on the screen. 
//				
//				
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/27	Tom			3.0			New
//
//----------------------------------------------------------------------
pBoolean DrawGraph(sn_WindowRef inWindowRef, double inH, double inV, double inGraphWidth, double inGraphHeight, double* inPotentials, long inNumPotentials)
{
	if (inPotentials == NULL || inNumPotentials < 0 || inNumPotentials > 1000)
		return false;
	
	// first look for the highest potential
	double highestPotential = inPotentials[0];
	long count;
	for (count = 0; count < inNumPotentials; count++)
	{
		if (inPotentials[count] > highestPotential)
			highestPotential = inPotentials[count];
	}
	
	if (highestPotential <= 0.0)
		return false;
	
	// now we need a scale factor:
	double vertScale = inGraphHeight*0.80/highestPotential;
	double horizontalStep = inGraphWidth/inNumPotentials;
	
	// Now draw the outline:
	//--------------------------
	TRGBColor theColor;
	theColor.red = 0.8;
	theColor.green = 0.2;
	theColor.blue = 0.0;
	
	OSMoveTo(inH, inV);
	OSLineTo(inH + inGraphWidth, inV, theColor);
	OSLineTo(inH + inGraphWidth, inV + inGraphHeight, theColor);
	OSLineTo(inH, inV + inGraphHeight, theColor);
	OSLineTo(inH, inV, theColor);
	
	
	// Now draw the data.
	// -----------------------
	theColor.red = 0.99; 
	theColor.green = 0.0; 
	theColor.blue = 0.1; 
	OSMoveTo(inH, inV + inGraphHeight);
	double currentH = inH;
	double baseV = inV + inGraphHeight;
	for (count = 0; count < inNumPotentials; count++)
	{
		double lineHeightPixels = inPotentials[count]*vertScale;
		OSLineTo(currentH, baseV - lineHeightPixels, theColor);
		currentH += horizontalStep;
		OSLineTo(currentH, baseV - lineHeightPixels, theColor);
	}
	
	return true; // graph drawn
}

//----------------------------------------------------------------------
//	DrawSETISpikes
//
//	Purpose:	Draws the spikes that have been found in the SETI 
//				
//				
//					
//	Inputs:		The window ref should always be as passed to the plugin in DoOperation(), with op_DrawLayer 
//
//				This call is only made if there is a good chance that the SETI information will be seen on the
//				screen. ie: only call if ChanceForSETIAreaToDraw returns true.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/27	Tom			3.0			New
//
//----------------------------------------------------------------------
pBoolean DrawSETISpikes(sn_WindowRef inWindowRef)
{
	PXYZCoordinates perpXYZ;
	PXYZCoordinates parallelXYZ;
	GetPerpAndParallelVectors(inWindowRef, perpXYZ, parallelXYZ);
	// The spikes are already read in from the file. 
	// We want to draw them here. Since the spikes vary in power,
	// we want to only draw the higher powered ones, such that the
	// SETI box is filled in nicely. Thus we draw more spikes as the user
	// zooms in. 
	// Also draw the spikes as longer if more powerful, with
	// colour to indicate the frequency of the spike.
	//------------------------------------------------
	
	// The spikes are sorted from lowest to highest power.
	// How to determine the cutoff? Perhaps showing 
	// a maximum of kMaxLinesPerInch on the screen is a good 
	// starting point. 
	const double kMaxLinesPerInch = 10;
	
	// Now, we need the onscreen size of the SETI region in order to 
	// find out how many spikes to draw.
	//-------------------------------------
	double theSETISizeRadians = gWorkUnit.angle_range*kDegreesToRadians;
	double windowFieldRadians;
	snErr error = sn_GetFieldOfView(inWindowRef, &windowFieldRadians);
	if (error != snNoErr)
		return false;
	
	// if a typical window is 1000 pixels wide, then the size in pixels
	// of the SETI region is:
	const double kRoughWindowSizePixels = 1000.0;
	double roughtSETISizePixels =	kRoughWindowSizePixels*theSETISizeRadians/windowFieldRadians;
	double roughtSETISizeInches =	roughtSETISizePixels/80.0;
	long numberOfSpikesToDraw = roughtSETISizeInches*kMaxLinesPerInch;
	
	if (numberOfSpikesToDraw > gNumSpikes)
		numberOfSpikesToDraw = gNumSpikes;
	
	// Since the SETI spikes are very often on top of one another, I 
	// draw a little horizontal line across the top of the bar.
	// we want the horiaontal bars to size independent of zoom state on the 
	// window. Figure this size out here, outside the loop.
	//----------------------------------------------------------
	const double kTopLineSizePixels = 10;
	double topLineSizeDegrees = kTopLineSizePixels/kRoughWindowSizePixels*windowFieldRadians*kRadiansToDegrees;
	
	
	// So now we know that we have to draw the stuff in the sky
	//-------------------------------------------------------------
	// Lets get going:
	for (long count = 0; count < numberOfSpikesToDraw; count++)
	{
		TSpike currentSpike = gSpikes[count];
		
		// we need the colour and size of the spike to draw. 
		// Note that we perhaps should use a constant scale (in radians)
		// to show the spikes, so that, when you run across a really big spike,
		// it will show up in the window as a large spike
		TRGBColor spikeColor;
		
		double spikeSizeDegrees = GetSpikeDrawSize(currentSpike);
		GetSpikeColor(currentSpike, spikeColor);
		
		// now we have to draw a line from the base of the SETI square to the spike
		// height.
		// Use the supplied perpendicular vector to set up the drawing direction
		// 
		PXYZCoordinates spikeLocationXYZ;
		RaDecToXYZ(currentSpike.ra, currentSpike.dec, spikeLocationXYZ);
		double factor = kSETIBeamWidthDeg*0.5*kDegreesToRadians; // we want a vector that is 0.05 degrees long, convert to radians for our normalized vector
	
		// now, we need the four points of the rectangle.
		PXYZCoordinates spikeBaseXYZ;
		AddAndNormalizeVectors(spikeLocationXYZ, perpXYZ, -kSETIBeamWidthDeg*0.5*kDegreesToRadians, spikeBaseXYZ);
		
		PXYZCoordinates spikeTopXYZ;
		AddAndNormalizeVectors(spikeBaseXYZ, perpXYZ, spikeSizeDegrees*kDegreesToRadians, spikeTopXYZ);

		XYZLineTo(inWindowRef, spikeBaseXYZ, spikeTopXYZ, spikeColor);
		
		// now we want to put the cap on the spike, so we can see the spikes all piled up on each other.
		PXYZCoordinates capLeftXYZ;
		AddAndNormalizeVectors(spikeTopXYZ, parallelXYZ, -topLineSizeDegrees*0.5*kDegreesToRadians, capLeftXYZ);
		PXYZCoordinates capRightXYZ;
		AddAndNormalizeVectors(capLeftXYZ, parallelXYZ, topLineSizeDegrees*kDegreesToRadians, capRightXYZ);
		
		XYZLineTo(inWindowRef, capLeftXYZ, capRightXYZ, spikeColor);
	}
	return true;
}

//----------------------------------------------------------------------
//	GetSpikeDrawSize
//
//	Purpose:	A spike with larger power draws bigger on the screen.
//				
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/28	Tom			3.0			New
//
//----------------------------------------------------------------------
double GetSpikeDrawSize(TSpike& inSpike)
{
	const double kPowerForFullRange = 1000.0;
	return kSETIBeamWidthDeg*(inSpike.power/kPowerForFullRange);
}

//----------------------------------------------------------------------
//	GetSpikeColor
//
//	Purpose:	Get a color so that we bluew shift spikes that are blue shifted.
//				
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/28	Tom			3.0			New
//
//----------------------------------------------------------------------
void GetSpikeColor(TSpike& inSpike, TRGBColor& outColor)
{
	// we look at the frequency of this spike as compared to the frequency of the centre of the
	// work unit, and the range.
	double freqRange = fabs(gWorkUnit.subband_center - gWorkUnit.subband_base);
	double unitlessFreq = (inSpike.freq - gWorkUnit.subband_center)/freqRange;
	
	// The frequency should be between -1 and 1
	//-------------------------------------------
	// Need 1 to map to blue, -1 to map to red.
	// 0 to map to white.
	//----------------------
	outColor.red = (1.0 - unitlessFreq)/2.0;
	outColor.green = 0.6;
	outColor.blue = (1.0 + unitlessFreq)/2.0;
	
	if (outColor.red > 1.0)
		outColor.red = 1.0;
	if (outColor.red < 0)
		outColor.red = 0;
	
	if (outColor.blue > 1.0)
		outColor.blue = 1.0;
	if (outColor.blue < 0)
		outColor.blue = 0;
}


//----------------------------------------------------------------------
//	DrawSETIOutline
//
//	Purpose:	Draws the outline of the SETI area on the screen.
//				
//					
//	Inputs:		The window ref should always be as passed to the plugin in DoOperation(), with op_DrawLayer 
//
//				This call is only made if there is a good chance that the SETI information will be seen on the
//				screen. ie: only call if ChanceForSETIAreaToDraw returns true.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
pBoolean DrawSETIOutline(sn_WindowRef inWindowRef, double& outMaxH, double& outMidV)
{
	// The SETI work unit is often a rectangle that is aligned with the ra-dec coord system.
	// this happens if the telescope was not slewing or trying to track an object in the sky.
	// If the telescope is slewing, then the path that it takes can be quite long. 
	// The path is always 0.1 degree wide, and it starts at the start coords, and goes over
	// to the end coords. 
	// ----------------------------------------
	TRGBColor theColor;
	theColor.red = 0.8;
	theColor.green = 0.2;
	theColor.blue = 0.0;

	// We need to determine the number of segments with which to draw the 
	// region. If the region will be drawing small, then we need only draw the 
	// start and end points.
	// If it is drawing large on the screen, we need to use more segments.
	// So what we do is find the size of the SETI region on the sky, then decide 
	// how many segments to draw.
	//-------------------------------
	
	// Attempt to make the code more robust:
	// If the number of segments we have appears goofy, if the 
	// first segment has an ra of 0, 
	// we assume that all of the reading of the segments from 
	// the work unit did not work, so we just draw one box segment,
	// the entire box.
	pBoolean problemWithCoordList = false;
	if (gWorkUnit.num_positions < 1 || gWorkUnit.num_positions > kMaxPositions || gWorkUnit.ra[0] == 0.0)
		problemWithCoordList = true;
	
	
	// Figure out how many segments we need.
	long skipAmount = 1;
	double numSegmentsNeeded = 1.0;
	if (!problemWithCoordList)
	{
		double pixelSize = GetPixelSizeOfSETIArea(inWindowRef);
		
		const double kMinSegmentPixels = 10.0;
		
		numSegmentsNeeded = pixelSize/kMinSegmentPixels;
		
		if (numSegmentsNeeded < 1.0)
			numSegmentsNeeded = 1.0;
		if (numSegmentsNeeded > gWorkUnit.num_positions)
			numSegmentsNeeded = gWorkUnit.num_positions;
		
		// we can only skip points so we now determine a skip factor
		//---------------------------------------------------------------
		skipAmount = gWorkUnit.num_positions/numSegmentsNeeded + 1;
	}
	
	PXYZCoordinates perpXYZ;
	PXYZCoordinates parallelXYZ;
	GetPerpAndParallelVectors(inWindowRef, perpXYZ, parallelXYZ);
	
	double leftTopH, leftTopV, leftBottomH, leftBottomV;
	double rightTopH, rightTopV, rightBottomH, rightBottomV;
	
	// get the 4 main window points, and use them to
	// figure out the position of the text display
	GetWindowPoints(inWindowRef, gWorkUnit.start_ra, gWorkUnit.start_dec, perpXYZ, leftTopH, leftTopV, leftBottomH, leftBottomV);
	GetWindowPoints(inWindowRef, gWorkUnit.end_ra, gWorkUnit.end_dec, perpXYZ, rightTopH, rightTopV, rightBottomH, rightBottomV);

	outMaxH = OurMax(OurMax(leftTopH, rightTopH), OurMax(leftBottomH, rightBottomH));
	outMidV = (leftTopV + leftBottomV + rightTopV + rightBottomV)/4.0;
	
	
	// if we only need a one segment box, then we draw it with one simple command
	if (numSegmentsNeeded < 2)
	{
		// draw the left edge
		OSMoveTo(leftTopH, leftTopV);
		OSLineTo(leftBottomH, leftBottomV, theColor);
		
		// top line
		OSMoveTo(leftTopH, leftTopV);
		OSLineTo(rightTopH, rightTopV, theColor);
		
		// bottom line
		OSMoveTo(leftBottomH, leftBottomV);
		OSLineTo(rightBottomH, rightBottomV, theColor);
		
		// right side
		OSMoveTo(rightTopH, rightTopV);
		OSLineTo(rightBottomH, rightBottomV, theColor);
	}
	else
	{
		// Draw the scene with all of the line segments.
		//----------------------------------------------
		// First, get the window points
		
		// Get the first two points, which I call left, as I imagine that we draw from left to right
		GetWindowPoints(inWindowRef, gWorkUnit.ra[1], gWorkUnit.dec[1], perpXYZ, leftTopH, leftTopV, leftBottomH, leftBottomV);
		
		// draw the left edge
		OSMoveTo(leftTopH, leftTopV);
		OSLineTo(leftBottomH, leftBottomV, theColor);
		
		// Now go through the points drawing the segments as we need them
		///------------------------------------------------------
		for (long count = 2; count < gWorkUnit.num_positions; count++)
		{
			GetWindowPoints(inWindowRef, gWorkUnit.ra[count], gWorkUnit.dec[count], perpXYZ, rightTopH, rightTopV, rightBottomH, rightBottomV);
			
			// top line
			OSMoveTo(leftTopH, leftTopV);
			OSLineTo(rightTopH, rightTopV, theColor);
			leftTopH = rightTopH;
			leftTopV = rightTopV;
			
			// bottom line
			OSMoveTo(leftBottomH, leftBottomV);
			OSLineTo(rightBottomH, rightBottomV, theColor);
			leftBottomH = rightBottomH;
			leftBottomV = rightBottomV;
		}
		
		OSMoveTo(rightTopH, rightTopV);
		OSLineTo(rightBottomH, rightBottomV, theColor);
	}
	return true; // worked
}

//----------------------------------------------------------------------
//	GetWindowPoints
//
//	Purpose:	Given two ra dec locations, comes up with
//				two window points that are suitable for framing.
//				Diagram:
//
//            perp
//				|		  wtop
//				|
//				|
//			--------      rdR
//
//
//   					  wbottom
//
//	The window points we get back are spaced at 
//				
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/03/01	Tom			3.0			New
//                                                                                                                                                                
//----------------------------------------------------- -----------------
void GetWindowPoints(sn_WindowRef inWindowRef, double inRa, double inDec, const PXYZCoordinates& inPerpXYZ, double& outTopH, double& outTopV, double& outBottomH, double& outBottomV)
{
	PXYZCoordinates theXYZ;
	RaDecToXYZ(inRa, inDec, theXYZ);
	
	double factor = kSETIBeamWidthDeg*0.5*kDegreesToRadians; // we want a vector that is 0.05 degrees long, convert to radians for our normalized vector
	
	// now, we need the four points of the rectangle.
	pBoolean inFront;
	PXYZCoordinates topXYZ;
	AddAndNormalizeVectors(theXYZ, inPerpXYZ, factor, topXYZ);
	snErr error = sn_XYZToWindowPoint(inWindowRef, kSNCelestialJ2000System, topXYZ, &outTopH, &outTopV, &inFront);

	PXYZCoordinates bottomXYZ;
	AddAndNormalizeVectors(theXYZ, inPerpXYZ, -factor, bottomXYZ);
	error = sn_XYZToWindowPoint(inWindowRef, kSNCelestialJ2000System, bottomXYZ, &outBottomH, &outBottomV, &inFront);
}


//----------------------------------------------------------------------
//	GetPerpAndParallelVectors
//
//	Purpose:	Figures out vectors that are perpendicular and parallel
//				to the SETI box.
//				
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/03/01	Tom			3.0			New
//                                                                                                                                                                
//----------------------------------------------------------------------
void GetPerpAndParallelVectors(sn_WindowRef inWindowRef, PXYZCoordinates& outPerpXYZVector, PXYZCoordinates& outParallelXYZVector)
{
	PXYZCoordinates startXYZ;
	RaDecToXYZ(gWorkUnit.start_ra, gWorkUnit.start_dec, startXYZ);
	PXYZCoordinates endXYZ;
	RaDecToXYZ(gWorkUnit.end_ra, gWorkUnit.end_dec, endXYZ);
	
	DoCrossProduct(startXYZ, endXYZ, outPerpXYZVector);
	NormalizeVector(outPerpXYZVector);
	
	// The vector parallel to the track is obtained by subtracting the vectors
	AddAndNormalizeVectors(startXYZ, endXYZ, -1.0, outParallelXYZVector);
}

//----------------------------------------------------------------------
//	DoCrossProduct
//
//	Purpose:	Does a vector cross product between two vectors, with the result going into the third
//				you can put the same vector in any of the slots, ie:
//				DoCrossProduct(a,b,a) is allowed.
//				
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/03/01	Tom			3.0			New
//                                                                                                                                                                
//----------------------------------------------------------------------
void DoCrossProduct(const PXYZCoordinates& inV1, const PXYZCoordinates& inV2, PXYZCoordinates& outResult)
{
	PXYZCoordinates v1 = inV1; // copy so that we can do DoCrossProduct(a,b,a)
	PXYZCoordinates v2 = inV2;

	outResult.x = v1.y*v2.z - v1.z*v2.y;
	outResult.y = v1.z*v2.x - v1.x*v2.z;
	outResult.z = v1.x*v2.y - v1.y*v2.x;
}
	
//----------------------------------------------------------------------
//	NormalizeVector
//
//	Purpose:	Makes the passed vector have length 1.
//				
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/03/01	Tom			3.0			New
//                                                                                                                                                                
//----------------------------------------------------------------------
void NormalizeVector(PXYZCoordinates& inV)
{
	if (isnan(inV.x))
		inV.x = 0.0;
	if (isnan(inV.y))
		inV.y = 0.0;
	if (isnan(inV.z))
		inV.z = 0.0;
	
	double_t	oneOverRadius,denom;
	
	denom = sqrt(inV.x*inV.x + inV.y*inV.y + inV.z*inV.z);
	if (denom == 0.0)
	{
#if debug_mode
		//Signal("\pAttempt to normalize a zero vector.");
#endif debug_mode
		inV.x = 0.0;
		inV.y = 0.0;
		inV.z = 1.0;
	}
	else
	{
		oneOverRadius = 1.0/denom;
		inV.x *= oneOverRadius;
		inV.y *= oneOverRadius;
		inV.z *= oneOverRadius;
	}
}
//----------------------------------------------------------------------
//	NormalizeVector
//
//	Purpose:	Makes the passed vector have length 1.
//				
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/03/01	Tom			3.0			New
//                                                                                                                                                                
//----------------------------------------------------------------------
void AddAndNormalizeVectors(const PXYZCoordinates& inV1, const PXYZCoordinates& inV2, double inFactorFor2, PXYZCoordinates& outResult)
{
	PXYZCoordinates v1 = inV1; // copy so that we can do DoCrossProduct(a,b,a)
	PXYZCoordinates v2 = inV2;

	outResult.x = v1.x + v2.x*inFactorFor2; 
	outResult.y = v1.y + v2.y*inFactorFor2; 
	outResult.z = v1.z + v2.z*inFactorFor2; 

	NormalizeVector(outResult);
}

//----------------------------------------------------------------------
//	RaDecLineTo
//
//	Purpose:	Does a line from one ra dec to another.
//				
//					
//	Inputs:		The ra, dec starting and ending points of the line and the colour. The ra dec are to be passed 
//				in degrees. 
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//                                                                                                                                                                
//----------------------------------------------------------------------
snErr RaDecLineTo(sn_WindowRef inWindowRef, double inStartRADeg, double inStartDecDeg, double inEndRADeg, double inEndDecDeg, TRGBColor inColor)
{
	PXYZCoordinates startXYZ;
	RaDecToXYZ(inStartRADeg, inStartDecDeg, startXYZ);
	PXYZCoordinates endXYZ;
	RaDecToXYZ(inEndRADeg, inEndDecDeg, endXYZ);

	return XYZLineTo(inWindowRef, startXYZ, endXYZ, inColor);
}

//----------------------------------------------------------------------
//	XYZLineTo
//
//	Purpose:	Draws a line from one point to the other.
//				The points are given in XYZ ra dec vector coords.
//					
//	Inputs:		The ra, dec starting and ending points of the line and the colour. The ra dec are to be passed 
//				in degrees. 
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//                                                                                                                                                                
//----------------------------------------------------------------------
snErr XYZLineTo(sn_WindowRef inWindowRef, PXYZCoordinates& inFrom, PXYZCoordinates& inTo, TRGBColor inColor)
{
	// we have to get the coordinates of the point in terms of the window
	double startH;
	double startV;
	pBoolean inFront;
	snErr error = sn_XYZToWindowPoint(inWindowRef, kSNCelestialJ2000System, inFrom, &startH, &startV, &inFront);
	if (!inFront || error != snNoErr)
		return snErr_ParameterError;
	
	double endH;
	double endV;
	error = sn_XYZToWindowPoint(inWindowRef, kSNCelestialJ2000System, inTo, &endH, &endV, &inFront);
	if (!inFront || error != snNoErr)
		return snErr_ParameterError;
	
	// now we have window points.
	// The Win32 and Mac drawing systems will not take any line that is drawn to a point that
	// is over 32k pixels from the origin.
	const long kMaxCoordinate = 32000;
	if (fabs(startH) > kMaxCoordinate || fabs(startV) > kMaxCoordinate || fabs(endH) > kMaxCoordinate || fabs(endV) > kMaxCoordinate)
		return snErr_ParameterError;
	
	// now we have everything.
	// we need to actually draw the line.
	// this is done differently on the two operating systems
	//-------------------------------------------------------
	OSMoveTo(startH, startV);
	OSLineTo(endH, endV, inColor);
	
	return snNoErr;
}

//----------------------------------------------------------------------
//	OSMoveTo
//
//	Purpose:	This does a moveto in the current drawing environment. 
//				On the Mac the port is set for you to the correct, usually offscreen world that 
//				we have. On the PC, I figure out the HDC that we are drawing into 
//				when opDrawLayer is called, and we use that.
//					
//	Inputs:		h,v, in pixels. Don't pass in numbers over 32k
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
void OSMoveTo(double inH, double inV)
{
#if TARGET_OS_MAC
	MoveTo(inH, inV);
#endif

#if TARGET_OS_WIN32
	MoveToEx(gCurrentDC, inH, inV, NULL);
#endif
}


//----------------------------------------------------------------------
//	OSDrawText
//
//	Purpose:	Draws text at the passed spot.
//					
//	Inputs:		h,v, in pixels. Don't pass in numbers over 32k
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
void OSDrawText(double inH, double inV, char* inTextString, const TRGBColor& inColor, pBoolean inUnderline)
{
#if TARGET_OS_MAC
	RGBColor theColor;
	theColor.red = inColor.red*65535;
	theColor.green = inColor.green*65535;
	theColor.blue = inColor.blue*65535;
	RGBForeColor(&theColor);
	TextSize(kTextSize);
	TextFont(1);
	TextMode(transparent);
	
	unsigned char pString[256];
	PStringAssign(pString, inTextString);
	MoveTo(inH, inV);
	
	if (inUnderline)
		TextFace(underline);
	DrawString(pString);
	
	TextFace(0);
#endif

#if TARGET_OS_WIN32
	// we have to get the color, and then draw the text.
	//-----------------------------------------------------
	COLORREF theTextColor = TRGBToColorRef(inColor);
	COLORREF savedTextColor = GetTextColor(gCurrentDC);
	SetTextColor(gCurrentDC, theTextColor);
	
	// Do this font dance
	LOGFONT		fontRecord;
	fontRecord.lfHeight			= -kTextSize; 	// negative is what we use
	fontRecord.lfWidth			= 0; 
	fontRecord.lfEscapement		= 0; 
	fontRecord.lfOrientation	= 0; 
	fontRecord.lfWeight			= FW_DONTCARE; 
	fontRecord.lfItalic			= false; 
	fontRecord.lfUnderline		= inUnderline; 
	fontRecord.lfStrikeOut		= false; 
	fontRecord.lfCharSet		= ANSI_CHARSET; // use DEFAULT_CHARSET to see what the character set really looks like
	fontRecord.lfOutPrecision	= OUT_DEFAULT_PRECIS; 
	fontRecord.lfClipPrecision	= CLIP_DEFAULT_PRECIS; 
	fontRecord.lfQuality		= DEFAULT_QUALITY; 
	fontRecord.lfPitchAndFamily	= DEFAULT_PITCH;
	strcpy(fontRecord.lfFaceName, "Arial");
	
	HFONT ourFont = CreateFontIndirect(&fontRecord);
	
	HFONT currentFont = NULL;
	if (ourFont != NULL)
		currentFont = (HFONT) SelectObject(gCurrentDC, ourFont);

	SetBkMode(gCurrentDC, TRANSPARENT);
	
	// In order to have the text in the same place for both Mac and Windows,
	// we need to adjust where the text will draw.
	// This makes the text draw so the base line is at the passed 
	// position, rather than the MS standard of text drawing under and to the
	// left of the passed position. 
	inV -= kTextSize;
	
	pBoolean worked = TextOut(gCurrentDC, inH, inV, inTextString, strlen(inTextString)); 

	if (currentFont != NULL)
		SelectObject(gCurrentDC, currentFont);
	DeleteObject(ourFont); 			// delete the font here, so no leaks
	SetTextColor(gCurrentDC, savedTextColor);
#endif

}

//----------------------------------------------------------------------
//	OSLineTo
//
//	Purpose:	This does a moveto in the current drawing environment. 
//				On the Mac the port is set for you to the correct, usually offscreen world that 
//				we have. On the PC, I figure out the HDC that we are drawing into 
//				when opDrawLayer is called, and we use that.
//					
//	Inputs:		h,v, in pixels. Don't pass in numbers over 32k
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
void OSLineTo(double inH, double inV, TRGBColor inColor)
{
#if TARGET_OS_MAC
	RGBColor theColor;
	theColor.red = inColor.red*65535;
	theColor.green = inColor.green*65535;
	theColor.blue = inColor.blue*65535;
	RGBForeColor(&theColor);
	LineTo(inH, inV);
#endif

#if TARGET_OS_WIN32
	// we have to create a pen of the passed color, and put it in the DC. 
	// then drawe the line!
	COLORREF thePenColor = TRGBToColorRef(inColor);
	HPEN thePen = CreatePen(PS_SOLID, 1, thePenColor);
	HPEN oldPen = (HPEN) SelectObject(gCurrentDC, thePen);
	LineTo(gCurrentDC, inH, inV);
	thePen = (HPEN) SelectObject(gCurrentDC, oldPen); // thePen we get back here should be the one we made.
	DeleteObject(thePen); 			// delete the pen here, so no leaks
#endif
}

//----------------------------------------------------------------------
//	GetPixelSizeOfSETIArea
//
//	Purpose:	Calculate the size in pixels of the SETI area
//				
//					
//	Inputs:		The window.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/03/01	Tom			3.0			New
//
//----------------------------------------------------------------------
double GetPixelSizeOfSETIArea(sn_WindowRef inWindowRef)
{
	PXYZCoordinates startXYZ;
	RaDecToXYZ(gWorkUnit.start_ra, gWorkUnit.start_dec, startXYZ);
	PXYZCoordinates endXYZ;
	RaDecToXYZ(gWorkUnit.end_ra, gWorkUnit.end_dec, endXYZ);
	
	double setiStartWindowH;
	double setiStartWindowV;
	double setiEndWindowH;
	double setiEndWindowV;
	pBoolean inFront;
	
	snErr error = sn_XYZToWindowPoint(inWindowRef, kSNCelestialJ2000System, startXYZ, &setiStartWindowH, &setiStartWindowV, &inFront);
	error = sn_XYZToWindowPoint(inWindowRef, kSNCelestialJ2000System, endXYZ, &setiEndWindowH, &setiEndWindowV, &inFront);
	
	double netH = fabs(setiStartWindowH - setiEndWindowH);
	double netV = fabs(setiStartWindowV - setiEndWindowV);
	return sqrt(netH*netH + netV*netV);
}

//----------------------------------------------------------------------
//	ChanceForSETIAreaToDraw
//
//	Purpose:	When we are called to draw the SETI information on screen,
//				we only want to do it if there is a chance that something will actually
//				show up. In other words we are called for every update, whether or not the
//				
//					
//	Inputs:		The window ref should always be as passed to the plugin in DoOperation(), with op_DrawLayer 
//
//				This call should be as fast as possible, since it will be called even for views that do not 
//				have the region of interest in them.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
pBoolean ChanceForSETIAreaToDraw(sn_WindowRef inWindowRef)
{
	// One trick for making sure code is fast is by not calling any system calls.
	// All drawing calls are 'slow' especially compared with a bit of math.
	//-----------------------------------------------------
	// now, we want to draw some fancy graphics, but only if there is a chance that they will show up on the screen.
	// also, we want to only draw the outline if we are zoomed out far. 
	// here we go. 
	
	// we need the middle of the SETI area.
	// Get the two end vectors, add them
	PXYZCoordinates startXYZ;
	RaDecToXYZ(gWorkUnit.start_ra, gWorkUnit.start_dec, startXYZ);
	PXYZCoordinates endXYZ;
	RaDecToXYZ(gWorkUnit.end_ra, gWorkUnit.end_dec, endXYZ);
	
	PXYZCoordinates theSETIXYZCentre;
	AddAndNormalizeVectors(startXYZ, endXYZ, 1.0, theSETIXYZCentre);
	
	// one thing we can check is whether or not the SETI stuff is anywhere near the screen.
	double setiCentreWindowH;
	double setiCentreWindowV;
	pBoolean inFront;
	
	// is the current point on the screen?
	snErr error = sn_XYZToWindowPoint(inWindowRef, kSNCelestialJ2000System, theSETIXYZCentre, &setiCentreWindowH, &setiCentreWindowV, &inFront);
	
	// first off, if the point is not even in front of the viewer, but rather around back, then we don't need to draw.
	if (inFront == false || error != snNoErr)
		return false;
	
	// if either of the coords is way out, also give up
	if (fabs(setiCentreWindowH) > 20000 || fabs(setiCentreWindowV) > 20000)
		return false;
	
	// so now we know that the SETI stuff might be near the screen.
	// we can work out the size of the screen, and the size of the SETI square to see if they intersect.
	PXYZCoordinates windowCenterXYZ;
	double windowFieldRadians;
	error = sn_GetWindowCentreCoords(inWindowRef, kSNCelestialJ2000System, &windowCenterXYZ);
	error = sn_GetFieldOfView(inWindowRef, &windowFieldRadians);
	
	// The size of the SETI area is rougly given by the angle_range.
	// if we use this angle_range as a radius, we will overestimate by 
	// 2 which is safe.
	double radianSizeOfSETI = gWorkUnit.angle_range*kDegreesToRadians;
	
	// So now we have two vectors, and we want to find out the angle between them.
	// use the dot product. The two vectors we have are normalized to length 1, 
	// so the dot product is equal to the cos() of the angle between them.
	// we already know that SETI stuff is in front, so this dot product should always be 
	// positive.
	double dotProd = windowCenterXYZ.x*theSETIXYZCentre.x + windowCenterXYZ.y*theSETIXYZCentre.y + windowCenterXYZ.z*theSETIXYZCentre.z;
	if (dotProd >= 0.999999999)
		return true; // they are very very close.
	
	double angleBetween = acos(dotProd);  // acos will choke on values of dotProd that are 1.0 or higher
	// now, we want to compare the angle between the centres of the window and the SETI region, in order to see if it will draw.
	if (angleBetween < (radianSizeOfSETI + windowFieldRadians))
		return true;
	else
		return false;
}

// Mac specific code:
//----------------------------------
//----------------------------------------------------------------------
//	ReadSETIFile (Mac version)
//
//	Purpose:	Given a passed name will find open and read the file into a buffer.
//					
//	Inputs:		inFileName: a 'c' string name of the file, without any path information. 
//	outputs:	ioBuffer: will be filled with null terminated content from the file. If there are 
//							null characters in the file (ie: it is not a text file) then the null 
//						termination will not be relevant. It seems though, that all of the SET 2.x files
//						are unix-style text files.
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
#if TARGET_OS_MAC
long ReadSETIFile(char* inFileName, char* ioBuffer, long inBufferSize)
{
	if (ioBuffer == NULL)
		return 0; // need a buffer
	
	// On the Mac SETI 2.0 files are located in the Application Support folder in the system 
	// folder, at least on OS 9. If we can't find app support, then look to the preferences.
	short appSupportVRefNum = 0;
	long appSupportDirID = 0;
	OSErr theError = FindFolder(kOnSystemDisk, kApplicationSupportFolderType, false, &appSupportVRefNum, &appSupportDirID);
	
	short prefsVRefNum = 0;
	long prefsDirID = 0;
	OSErr prefsError = FindFolder(kOnSystemDisk, kPreferencesFolderType, false, &prefsVRefNum, &prefsDirID);
	
	if (theError != noErr && prefsError != noErr)
		return 0; // we read nothing, could not find the folder 
	
	// so now we have the folder that the SETI folder is in.
	// Look for the SETI folder, it is called "SETI@home Data", at least fo verison 2.0 on the iMac here.
	char pathANDFile[256] = ":SETI@home Data:";
	strcat(pathANDFile, inFileName);
	
	unsigned char pascalPathAndFile[256];
	PStringAssign(pascalPathAndFile, pathANDFile);
	
	FSSpec theSETIFileDescription;
	theError = FSMakeFSSpec(appSupportVRefNum,appSupportDirID, pascalPathAndFile, &theSETIFileDescription); // pstrings start with \p
	if (theError != noErr)
	{
		// Try the prefs folder
		theError = FSMakeFSSpec(prefsVRefNum,prefsDirID, pascalPathAndFile, &theSETIFileDescription); // pstrings start with \p
	}
	
	if (theError != noErr)
		return 0; // we read nothing, could not find the seti files folder or perhaps the file we are looking for
		
	// oK, so now we know that the file we want is there. We need to open it and read it into the passed buffer.
	//------------------------------------------
	short fileReference = 0;
	theError = FSpOpenDF(&theSETIFileDescription, fsRdPerm, &fileReference);
	if (theError != noErr)
		return 0; // we read nothing, could not open the file for read access.

	// set the marker to the start of the file
	SetFPos(fileReference, fsFromStart, 0);
	long fileLength = 0;
	GetEOF(fileReference, &fileLength);
	long amountToRead = inBufferSize - 1;
	if (fileLength < (inBufferSize - 1))
		amountToRead = fileLength;
		
	theError = FSRead(fileReference, &amountToRead, ioBuffer);
	FSClose(fileReference);
	ioBuffer[amountToRead] = NULL; // null terminate
	if (theError != noErr)
	{
		ioBuffer[0] = NULL; // null terminate to zero length
		return 0; // The read did not work, so we do nothing
	}
	
	return amountToRead; // return the number of byts read into the buffer, not including null terminator, as is usual in C. 
}					
#endif



// Windows specific code:
//----------------------------------
//----------------------------------------------------------------------
//	FindSETIFilesFromRegistry (windows version)
//
//	Purpose:	We look for the path to the SETI files by looking in the registry.
//					
//	Inputs:		ioPath: 
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
#if TARGET_OS_WIN32
static snErr FindSETIFilesFromRegistry(char* ioPath)
{
	// open the key containing the saved first used date
	HKEY	theKey;
   // DWORD	disposition;
	//long	status = RegCreateKeyEx(HKEY_CURRENT_USER, "Software\SETI@Home", 0, "", REG_OPTION_NON_VOLATILE, KEY_QUERY_VALUE, NULL, &theKey, &disposition);
	long	status = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\SETI@Home", 0, KEY_READ, &theKey);
	if (status != ERROR_SUCCESS)
	{
		// The registry info for SETI seems to be in a different location for Win98.
		status = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\SETI@Home", 0, KEY_READ, &theKey);
	}
	
	if (status == ERROR_SUCCESS) 
	{
		// read the value of the the client path
		unsigned long	keyType;
		unsigned long	keyValueSize = MAX_PATH; // Assume that ioPath is MAX_PATH long!
		status = RegQueryValueEx(theKey, "ClientDir", NULL, &keyType, (LPBYTE) ioPath, &keyValueSize);
		if (status == ERROR_SUCCESS) 
		{
			return snNoErr;
		}
	}
	
	return snErr_Failed; //
}					
#endif

//----------------------------------------------------------------------
//	ReadSETIFile (windows version)
//
//	Purpose:	Given a passed name will find open and read the file into a buffer.
//					
//	Inputs:		inFileName: a 'c' string name of the file, without any path information. 
//	Inputs:		inBuffer: 
//
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/02/23	Tom			3.0			New
//
//----------------------------------------------------------------------
#if TARGET_OS_WIN32
long ReadSETIFile(char* inFileName, char* inBuffer, long inBufferSize)
{
	// On windows, we locate the files that SETI is currently using with the registry (I think)
	char theSETIPath[MAX_PATH] = "";
	snErr theError = FindSETIFilesFromRegistry(theSETIPath);
	if (theError != snNoErr)
		return 0;
		
	
	strcat(theSETIPath, "\\"); // add path sep
	strcat(theSETIPath, inFileName); // add name of file
	DWORD fileSharePerm = FILE_SHARE_READ + FILE_SHARE_WRITE;
	HANDLE fileHandle = CreateFile(theSETIPath, GENERIC_READ, fileSharePerm, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (fileHandle == NULL)
		return 0;
	
	// now read and close the file.
	//------------------------------
	DWORD fileSizeUpper = 0; // will stay 0 for files under 2GB
	DWORD fileLength = GetFileSize(fileHandle, &fileSizeUpper);
	SetFilePointer(fileHandle, 0, 0, FILE_BEGIN);
	
	unsigned long bytesToRead = inBufferSize - 1;
	if (fileLength < bytesToRead)
		bytesToRead = fileLength;

	pBoolean worked = ReadFile(fileHandle, inBuffer, bytesToRead, &bytesToRead, NULL);
	CloseHandle(fileHandle);
	
	inBuffer[bytesToRead] = NULL;
	
	if (worked)
	{
		inBuffer[bytesToRead] = NULL;
		return bytesToRead;
	}
	else
	{
		inBuffer[0] = NULL;
		return 0;
	} 
}					
#endif



//----------------------------------------------------------------------
//	DLLMain
//
//	Purpose:	DLL initialization called first thing
//					
//	Inputs:		ResourceID		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Cross platform SETI plugin
//
//----------------------------------------------------------------------
#if TARGET_OS_WIN32
extern "C" BOOL WINAPI DllMain ( HINSTANCE hInst, DWORD wDataSeg, LPVOID lpvReserved );

BOOL WINAPI DllMain( HINSTANCE hInst, DWORD wDataSeg, LPVOID lpReserved )
{
	
#if debug_mode
	// when debugging, we put up a dialog to say that we have been called.
	//MessageBox(GetFocus(), "DLLMain Called", "SETI - plugin", MB_OK|MB_SYSTEMMODAL);
#endif
	
	// we need to get the HINSTANCE of this module
	switch(wDataSeg)
	{
    	case DLL_PROCESS_ATTACH:
			gDLLInstance = hInst;
	  		break;
	}
	
	return 1;
}
#endif

// These macros are in the Win32 headers as well.
#if TARGET_OS_WIN32
// same as windows RGB only x-platform
#define SiennaRGB(r,g,b)	((DWORD)(((unsigned char)(r)|((WORD)((unsigned char)(g))<<8))|(((DWORD)(unsigned char)(b))<<16)))
#define SiennaPALETTERGB(r,g,b)   (0x02000000 | SiennaRGB(r,g,b))
#endif


//----------------------------------------------------------------------
//	RGBToColorRef
//
//	Purpose:	converts to a windows style colour  
//				
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	97/09/15	Tom			3.0.0		New
//
//---------------------------------------------------------------------- 
#if TARGET_OS_WIN32
unsigned long TRGBToColorRef(const TRGBColor& inColour)
{
	const double kFloatToColorRefValue = 256.0;
	unsigned long outColorRef = 0;
	// RGB needs values that are between 0-255
	// We add on 0x80 (1/2) in order to round the colour
	// we have to check for overflow, too
	unsigned long red = inColour.red*kFloatToColorRefValue;
	unsigned long green = inColour.green*kFloatToColorRefValue;
	unsigned long blue = inColour.blue*kFloatToColorRefValue;
	
	// Use Palette RGB colous, since that way the GDI always does the 
	// color match under 8bit , whereas with just a RGB the device does the
	// conversion, and devices do things differently
	outColorRef = SiennaPALETTERGB(red, green, blue);
	return outColorRef; 
}
#endif


//----------------------------------------------------------------------
//	ColorRefToTRGB
//
//	Purpose:	converts from a windows style colour  
//				
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	97/09/15	Tom			3.0.0		New
//
//---------------------------------------------------------------------- 

#if TARGET_OS_WIN32
void ColorRefToTRGB(unsigned long inColorRef, TRGBColor& outColour)
{
	const double kColorRefValueToFloat = 1.0/256.0;
	const unsigned long kRedMask =   0x000000ff;
	const unsigned long kGreenMask = 0x0000ff00;
	const unsigned long kBlueMask =  0x00ff0000;
	unsigned long aRed		= (inColorRef & kRedMask);
	unsigned long aGreen	= (inColorRef & kGreenMask) >> 8;
	unsigned long aBlue	= (inColorRef & kBlueMask) >> 16;

	outColour.red	= aRed*kColorRefValueToFloat;
	outColour.green	= aGreen*kColorRefValueToFloat;
	outColour.blue	= aBlue*kColorRefValueToFloat;
}
#endif

