//----------------------------------------------------------------------
//	Copyright  1990-1998 by Sienna Software, Inc., All rights reserved.
//
//	File:		Controller.cpp
//
//	Contains:	Skelton code for a Starry Night telescope controller plug-in. Use this
//				outline to create a telescope controller that can be used with
//				Starry Night Basic and Starry Night Deluxe.
//
//				Starry Night communicates with your plug-in through the DoOperation
//				function (see description below). New with version 3 of Starry Night
//				is the ability for the plugin you write to call Starry Night.
//
//
//
//				Starry Night provides a "Telescope" menu when we return
//				fIsTelescopeController as true when we get called with
//				op_Initialize. Our plug-in is called with one of the
//				following op codes whenever the user selects one of the Telescope
//				menu's items:
//
//					Telescope
//					------------
//					Configure...				op_TelescopeConfigure
//					Update Location				op_TelescopeLocation
//					Update Time					op_TelescopeTime
//					Sync to Current Gaze		op_TelescopeSync
//					------------
//					Open Connection				op_TelescopeOpen
//					Open Connection				op_TelescopeClose
//					------------
//					Slew to Current Gaze		op_TelescopeSlew
//					Follow Telescope			op_TelescopeTrack
//
//				This example plug-in also adds two menu items to Starry Night's
//				contextual popup menus that appear whenever a user right-clicks
//				on an item. These two menu items are "Slew" and "Synch".
//
//				All the code that you need to implement is marked with
//				"<Insert your code here>".
//	
//	Authors:	Ted Leckie, Tom Andersen
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//	99/02/01	Tom			3.0			Version 3 offers callbacks, also x-platform plug-ins
//
//----------------------------------------------------------------------

// we have to figure out whether we are on macintosh power pc or windows.
// This bit of muck will need to be changed if you use compilers other than those by 
// Metrowerks (www.metrowerks.com) or Microsoft.
// If you are only building for Windows or only for Mac you can just set these
// For instance with a Windows only plug-in define TARGET_OS_WIN32 as true and
// TARGET_OS_MAC as false. (StarryNightPlugins.h uses these defines since it can be used
// on both Macintosh and Windows).
#if defined(_M_MPPC)	/* Visual C++ with Macintosh PowerPC target */
	#define TARGET_OS_MAC 1
	#define TARGET_OS_WIN32 0
#elif defined(_M_IX86)	/* Visual C++ with Intel x86 target */
	#define TARGET_OS_MAC 0
	#define TARGET_OS_WIN32 1
#elif defined(macintosh) /* Metrowerks for Macintosh */
	#define TARGET_OS_MAC 1
	#define TARGET_OS_WIN32 0
#elif defined(__INTEL__) /* Metrowerks for Intel*/
	#define TARGET_OS_MAC 0
	#define TARGET_OS_WIN32 1
#else
	you must define TARGET_OS_MAC and TARGET_OS_WIN32
#endif

// now add any header files we need to
#if TARGET_OS_WIN32
#include <windows.h>
#endif

// On the mac, I added system headers via a pre-compiled header option. 

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "StarryNightPlugins.h"

#define noError		0

const unsigned long	Command_Slew		= 20000;
const unsigned long	Command_Synch		= 20001;

// global variables
//-------------------------
PCentreAction	gCentreAction;

// For our example we keep a flag telling us whether a connection has been opened with
// the telescope or not. You are free to track state in whatever way makes sense for you.
pBoolean gConnectionOpened = false;

// We also keep a flag that tells us whether we are in tracking mode or not.
pBoolean gIsTracking = false;


#if TARGET_OS_MAC
// Starry Night will open the resource fork of this plugin if there is one.
// If you are using resources, you can make sure you have the right resource fork open by 
// setting this up
long gResourceFork = -1;	 
#endif


// function prototypes
double ClockSec(void);
void NoteToUser(long inResourceID);
unsigned char *PStringAssign(unsigned char *destination, const char *source);
void GetTelescopeRaDec(double* outRightAscension, double* outDeclination);
double GetTelescopeFieldOfView(void);
void DoInitialize(PInitialize* outInitRecord);
void DoMenuStatus(PMenuStatus* ioMenuRecord);
void DoTelescopeConfigure(void);
void DoTelescopeTime(const PTime* inTime);
void DoTelescopeLocation(const PLocation* inLocation);
void DoTelescopeOpen(void);
void DoTelescopeClose(void);
void DoTelescopeTrack(void);
void SlewTelescope(double inRightAscension, double inDeclination);
void SynchTelescope(double inRightAscension, double inDeclination);
void DoIdle(PEvent* ioEvent);
void DoDrawIndicator(PIndicator* outIndicator);
void DoBuildContextualPopup(const PBuildContext* inBuildContext);
void DoContextualPopup(const PDoContext* inContext);
void TakeWhirlWindTour(sn_WindowRef inWindowRef);
void HandleError(long inSNErr);
void CallBackErrorHandler(long inSNErr);
void DoCallBackAPIDemo(void);

// The protype for DoOperation is supplied by Sienna in Starry Night Plugins.h

// here are the resource numbers for dialogs for the DLL
const long kUpdateTelescopeTime = 500;
const long kUpdateTelescopeLocation = 501;
const long kOpenTelescopeConnection = 502;
const long kCloseTelescopeConnection = 503;
const long kSlewTelescope = 504;
const long kSynchTelescope = 505;
const long kSettingsDialog = 506;

const double kPiDefine	= 3.14159265358979323846;	// pi base unit used to calculate others
const double kDegreesToRadians = kPiDefine/180.0;
const double kAuToMeters = 1.4959787066e11;			// number of metres per astronomical unit
const double kMetersToAu = 1.0/kAuToMeters;			// number of astronomical unit per metres
const double kLightYearsPerParsec	=3.261633;					// Pete got this number from somewhere
const double kParsecsPerAu 			= 4.848136811e-6;			// explanatory suppl, page 716
const double kAuPerParsec 			= 1.0/kParsecsPerAu;		// explanatory suppl, page 716
const double kAuPerLightYear 		= kAuPerParsec/kLightYearsPerParsec;	
const double kLightYearPerAu 		= 1.0/kAuPerLightYear;		
const double kMetrePerLightYear		=kAuPerLightYear*kMetersToAu;


#if TARGET_OS_WIN32
HINSTANCE gDLLInstance;
//----------------------------------------------------------------------
//	DLLMain
//
//	Purpose:	DLL initialization called first thing
//					
//	Inputs:		ResourceID		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Cross platform telescope controller
//
//----------------------------------------------------------------------
extern "C" BOOL WINAPI DllMain ( HINSTANCE hInst, DWORD wDataSeg, LPVOID lpvReserved );

BOOL WINAPI DllMain( HINSTANCE hInst, DWORD wDataSeg, LPVOID lpReserved )
{
	
	::MessageBox(::GetFocus(), "DLLMain Called", "Telescope Controller", MB_OK|MB_SYSTEMMODAL);
	
	// we need to get the HINSTANCE of this module
	switch(wDataSeg)
	{
    	case DLL_PROCESS_ATTACH:
			gDLLInstance = hInst;
	  		break;
	}
	
	return 1;
}

#endif

//----------------------------------------------------------------------
//	ClockSec
//
//	Purpose:	returns time since computer started or some other arbitrary start, in seconds 
//					
//	Inputs:		ResourceID		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Cross platform telescope controller
//
//----------------------------------------------------------------------

double ClockSec(void)
{
	double rawTime = clock();
	double timeSec = rawTime/CLOCKS_PER_SEC;
	return timeSec;
}


//----------------------------------------------------------------------
//	NoteToUser
//
//	Purpose:	sends a message to the user
//					
//	Inputs:		ResourceID		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Cross platform telescope controller
//
//----------------------------------------------------------------------

void NoteToUser(long inResourceID)
{
#if TARGET_OS_MAC
	// interpret the resource as the number of an ALRT resource ID
	long savedResFile = ::CurResFile();
	::UseResFile(gResourceFork);
	OSErr error = ::NoteAlert(inResourceID, NULL);
	::UseResFile(savedResFile);
#endif

#if TARGET_OS_WIN32
	char message[256];
	int worked = ::LoadString(gDLLInstance, inResourceID, message, 255);
	::MessageBox(::GetFocus(), message, "Telescope Controller", MB_OK|MB_SYSTEMMODAL);
#endif
}




//----------------------------------------------------------------------
//	PStringAssign
//
//	Purpose:	assigns a c string to a pascal string.
//					
//	Inputs:		source			source string
//				destination		destination string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

unsigned char *PStringAssign(unsigned char *destination, const char *source)
{
	destination[0] = strlen(source);
	for (short charPos = destination[0] + 1; charPos--; destination[charPos+1] = source[charPos])
		;
	return (destination);
}

//----------------------------------------------------------------------
//	GetTelescopeRaDec
//
//	Purpose:	returns the ra/dec of where the telescope is currently pointed.
//					
//	Inputs:		outRightAscension	RA in epoch of current date.
//				outDeclination		Dec in epoch of current date.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void GetTelescopeRaDec(double* outRightAscension, double* outDeclination)
{
	// You need to query the telescope about its current viewing direction here.
	// <Insert your code here>
	*outRightAscension = 0.0;
	*outDeclination = 0.0;
}

//----------------------------------------------------------------------
//	GetTelescopeFieldOfView
//
//	Purpose:	returns the ra/dec of where the telescope is currently pointed.
//					
//	Outputs:	double		field of view in radians
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

double GetTelescopeFieldOfView(void)
{
	// You need to query the telescope about its current field of view here
	// or, if you can't obtain this information, you might want to have the
	// user input it in your settings dialog.
	// <Insert your code here>
	return 0.0524;		// approximately 3 degrees.
}

//----------------------------------------------------------------------
//	DoInitialize
//
//	Purpose:	Starry Night calls our plug-in with op_Initialize at when it is
//				starting up. We inform Starry Night about our needs and wants.
//				We could also perform one time memory allocations, etc here.
//					
//	Inputs:		ioParams	we must fill out this parameter block to inform
//							Starry Night of our needs.
//							If for any reason our initialization fails, we need
//							to set ioParams->fSuccess false.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoInitialize(PInitialize* outInitRecord)
{
#if TARGET_OS_MAC
	gResourceFork = ::CurResFile();
#endif

	// Starry Night passes us in the outInitRecord->fPluginAPI which version of the
	// plug-in API it supports. We require kVersion3Plugin or later so we check
	// that this is available. If it isn't then we return with fSuccess as false,
	// which causes our plug-in not to be used.
	if (outInitRecord->fPluginAPI < kVersion3Plugin)
	{
		outInitRecord->fSuccess = false;
		return;
	}
	outInitRecord->fPluginAPI = kVersion4Plugin;		// this is the API we use
	outInitRecord->fSuccess = true;
	
	// Plug-ins can request that Starry Night place a menu items in its "Settings"
	// menu for it. Since we are are a telescope controller we don't need a
	// special Settings menu item since the "Telescope" menu provides one.
	outInitRecord->fWantsMenuItem = false;
	outInitRecord->fOptionsMenuItem[0] = 0;		// text of menu item, if we wanted one

	// Plug-ins can also request that Starry Night create a floating window that
	// the plug-in can use. Since our telescope controller uses the "Telescope"
	// menu for its user interface, we don't need a floating window.
	outInitRecord->fWantsWindow = false;
	outInitRecord->fWindowMenuItem[0] = 0;		// if we had a window, name of menu item for opening and closing it

	// If you do any drawing into a Starry Night scene you need to inform
	// Starry Night at which "layer" your drawing will take place. You can
	// think of layers as onion skins in which drawing takes place. So to draw
	// something immediately after constellations are drawn but before the
	// ecliptic guides are drawn then you would set outInitRecord->fLayer to a
	// value between Depth_Constellations and Depth_EclipticGuides.
	// If your plug-in doesn't do any drawing (most don't) then set fLayer
	// as Depth_None.
	// Our telescope controller plug-in gets Starry Night to draw a field
	// of view indicator at the current location of the telescope so we
	// pass back that we want drawing to occur at Depth_Telrad.
	outInitRecord->fLayer = Depth_Telrad;		// pass back Depth_None if you don't do any drawing

	// We pass back that we are a telescope controller. This causes Starry Night
	// to provide a "Telescope" menu that provides the user interface for
	// interacting with our plug-in. Starry Night only supports one telescope
	// controller plug-in at a time and only communicates with the first it
	// encounters. 
	outInitRecord->fIsTelescopeController = true;

	// If you want a menu item for an About Box in Starry Night then set it here.
	// You will be called with an op_About everytime the user selects your menu
	// item.
	outInitRecord->fAboutMenuItem[0] = 0;	// we don't bother with an About Box

	// If you want to be called periodically by Starry Night everytime there is an
	// idle moment (this can be quite frequently) then you should return fWantsIdleTime
	// as true. For instance you could use this idle time to track a telescope that
	// is slewing from one location to another by drawing a moving field of view
	// indicator.
	outInitRecord->fWantsIdleTime = true;	// if true then plug-in receives op_Idle regularly
}

//----------------------------------------------------------------------
//	DoMenuStatus
//
//	Purpose:	Starry Night queries us to determine which Telescope menu items
//				should be enabled/disabled, checked/unchecked.
//					
//	Inputs:		ioMenuRecord	structure that informs us which command we are being
//								queried on. We also return status information in
//								this record.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoMenuStatus(PMenuStatus* ioMenuRecord)
{
	// Most of the menu items don't use checkmarks (the only exception is the
	// "Follow Telescope" (op_TelescopeTrack) item.
	ioMenuRecord->fHasCheck = false;

	// We set fIsEnabled true for each menu item we support.
	switch (ioMenuRecord->fMenuCommand)
	{
		case cmd_TelescopeConfigure:
			ioMenuRecord->fIsEnabled = true;
			break;
		case cmd_TelescopeTime:
			ioMenuRecord->fIsEnabled = gConnectionOpened;
			break;
		case cmd_TelescopeLocation:
			ioMenuRecord->fIsEnabled = gConnectionOpened;
			break;
		case cmd_TelescopeOpen:
			// if the connection to the telescope is already open then we
			// disable the "Open Connection" menu item.
			ioMenuRecord->fIsEnabled = !gConnectionOpened;
			break;
		case cmd_TelescopeClose:
			// We only enable the "Close Connection" menu item if we already
			// have an open connection.
			ioMenuRecord->fIsEnabled = gConnectionOpened;
			break;
		case cmd_TelescopeTrack:
			// If your telescope controller supports tracking then you should
			// "check" this menu item when tracking is in effect, and uncheck
			// it when it isn't.
			ioMenuRecord->fIsEnabled = gConnectionOpened;
			ioMenuRecord->fHasCheck = gIsTracking;
			break;
		case cmd_TelescopeSlew:
			// If we are tracking then by definition the telescope is already slewed
			// to the Starry Night gaze vector since Starry Night is supposed to be
			// tracking the telescope. Hence we disable this menu command when we
			// are tracking.
			ioMenuRecord->fIsEnabled = gConnectionOpened && !gIsTracking;
			break;
		case cmd_TelescopeSync:
			ioMenuRecord->fIsEnabled = gConnectionOpened;
			break;
	}
}

//----------------------------------------------------------------------
//	DoTelescopeConfigure
//
//	Purpose:	Your telescope controller may or may not have configuration options
//				(for example which communications port to use, etc). If you do
//				support various configurations you should show your settings
//				dialog everytime DoTelescopeConfigure is called.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoTelescopeConfigure(void)
{
	// <Insert your code here>
	::NoteToUser(kSettingsDialog);
}

//----------------------------------------------------------------------
//	DoTelescopeTime
//
//	Purpose:	The user selects "Update Time" from the "Telescope" menu to tell
//				the telescope what time it is. Starry Night passes
//				the time of the current view.
//					
//	Inputs:		inTime		a record describing the current time.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoTelescopeTime(const PTime* inTime)
{
	// <Insert your code here>
	NoteToUser(kUpdateTelescopeTime);
}

//----------------------------------------------------------------------
//	DoTelescopeLocation
//
//	Purpose:	The user selects "Update Location" from the "Telescope" menu to tell
//				the telescope where it is located. Starry Night passes
//				the location of the current view.
//					
//	Inputs:		inLocation	record describing the current location.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoTelescopeLocation(const PLocation* inLocation)
{
	// <Insert your code here>
	NoteToUser(kUpdateTelescopeLocation);
}

//----------------------------------------------------------------------
//	DoTelescopeOpen
//
//	Purpose:	The user selects "Open Connection" to initiate the connection
//				between the computer and the telescope. When you get called here
//				you should find the telescope and initiate a connection.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoTelescopeOpen(void)
{
	gConnectionOpened = true;
	// <Insert your code here>
	NoteToUser(kOpenTelescopeConnection);
}

//----------------------------------------------------------------------
//	DoTelescopeClose
//
//	Purpose:	The user selects "Close Connection" to terminate the connection
//				between the computer and the telescope. When you get called here
//				you should find the telescope and close the connection. We
//				disable the telescope menu items when the connection is closed.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoTelescopeClose(void)
{
	gConnectionOpened = false;
	// <Insert your code here>
	NoteToUser(kCloseTelescopeConnection);
	
}

//----------------------------------------------------------------------
//	DoTelescopeTrack
//
//	Purpose:	When the user puts our telescope controller into tracking mode then
//				it becomes our respondibility to keep Starry Night's view aligned with
//				the telescope. We do that when we receive an op_Idle event. See DoIdle
//				for more details.
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoTelescopeTrack(void)
{
	gIsTracking = !gIsTracking;
}

//----------------------------------------------------------------------
//	SlewTelescope
//
//	Purpose:	Slew the telescope to the passed gaze vector whenever you receive
//				op_TelescopeSlew. When the user selects "Slew to Current Gaze" from
//				the telescope menu they want the attached telescope to slew to the
//				same spot as the centre of the Starry Night screen. This is the gaze
//				vector that is passed with op_TelescopeSlew.
//					
//	Inputs:		inRightAscension	using epoch of current date (in radians)
//				inDeclination		using epoch of current date (in radians)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void SlewTelescope(double inRightAscension, double inDeclination)
{
	// <Insert your code here>
	NoteToUser(kSlewTelescope);
}

//----------------------------------------------------------------------
//	SynchTelescope
//
//	Purpose:	Synch the telescope to the passed gaze vector whenever you receive
//				op_TelescopeSync. When the user selects "Sync to Current Gaze" from
//				the telescope menu they want the attached telescope to synch to the
//				same spot as the centre of the Starry Night screen. This is the gaze
//				vector that is passed with op_TelescopeSync.
//					
//				For example, the user may centre on Altair, locate it in their scope and
//				then select "Sync to Current Gaze". It is our responsibility to inform
//				our telescope where it is currently pointed. This is referred to as
//				synch-ing the telescope.
//					
//	Inputs:		inGaze		vector to point at
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void SynchTelescope(double inRightAscension, double inDeclination)
{
	// <Insert your code here>
	NoteToUser(kSynchTelescope);
}

//----------------------------------------------------------------------
//	DoIdle
//
//	Purpose:	Starry Night calls our plug-in with op_Idle during idle moments. If we
//				are "tracking" the telescope (ie. Starry Night is acting as a slave to
//				the telescope) then we use this idle time to re-centre the Starry Night
//				view to the same position as the telescope.
//				Since op_Idle events come fast and furious we only re-centre Starry
//				Night's view once a second.
//					
//	Inputs:		ioEvent		we use this paramater block to tell Starry Night to
//							recentre its view when we are tracking the telescope.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoIdle(PEvent* ioEvent)
{
	// As default we will update Starry Night's view every second (1000 milliseconds) to follow
	// the telescope when tracking. You may wish to provide an option for changing this in your
	// settings dialog.
	const	double	kSecondsBetweenUpdates = 1.0;
	static	double	lastIdle = 0;

	if (gIsTracking && ((::ClockSec() - lastIdle) > kSecondsBetweenUpdates))
	{
		lastIdle = ::ClockSec();
		
		ioEvent->fOutOperation = out_CentrePoint;
		GetTelescopeRaDec(&gCentreAction.fRightAscension, &gCentreAction.fDeclination);
		// we override the user's pan setting and tell Starry Night to immediately centre
		// the ra/dec that we are going to pass back instead of slowly panning over to it.
		// If you prefer you can pan back to the telescope position by passing fPan as true.
		gCentreAction.fUseCustomPan = true;
		gCentreAction.fPan = false;
		// we let the user select the field of view. If instead you wanted to set the FOV then
		// you would pass back fAdjustFOV as true where fNewFOV contains the new FOV in radians.
		gCentreAction.fAdjustFOV = false;
		ioEvent->fOutParams = &gCentreAction;
	}
	
	// test the callback API
	static long staticCount = 0;
	if (staticCount < 1)
	{
		::DoCallBackAPIDemo();
		
		staticCount ++;
	}
	
	// <Insert your code here>
	// If you want to track a slew in progress you will want to insert code like this here:
	//	if (telescopeIsSlewing)
	//	{
	//		// have Starry Night update the view, which will then call DoDrawIndicator which shows
	//		// the telescope's current location
	//		ioEvent->fOutOperation = out_Update;
	//		ioEvent->fOutParams = NULL;
	//	}
}

//----------------------------------------------------------------------
//	GetNextElevation
//
//	Purpose:	gets elevation for the massive liftoff below
//					
//	Inputs:		inSNErr	description of possible error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/18	Tom			3.0			New Callback API
//
//----------------------------------------------------------------------
static double GetNextElevation(double inElevationNow)
{
	const double kElevationControl1 = 1.0; // lowering this number means the liftoff curve is flatter
	const double kElevationControl2 = 1.0/20.0; // raising this number controls how early the elevation slowdown factor sets in
	
	const double kMaximumElevation = 10000.0*kAuPerLightYear; //lets lift off to 10,000 ly
	double unitlessElevation = inElevationNow/kMaximumElevation;
	unitlessElevation *= kElevationControl1;
	
	double factor = 1.0 - pow(unitlessElevation/(1.0 + unitlessElevation), kElevationControl2);
	
	factor *= 0.2;
	
	double elevationStep = inElevationNow*factor;
	
	return elevationStep + inElevationNow;
}


//----------------------------------------------------------------------
//	TakeWhirlWindTour
//
//	Purpose:	I take the user on a wild ride
//					
//	Inputs:		inSNErr	description of possible error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/18	Tom			3.0			New Callback API
//
//----------------------------------------------------------------------
void TakeWhirlWindTour(sn_WindowRef inWindowRef)
{
	snErr error = ::sn_GotoFixedLocation(inWindowRef);
	
	pRect windowRect;
	windowRect.top = 50;
	windowRect.bottom = 450;
	windowRect.left = 50;
	windowRect.right = 650;
	
	error = ::sn_SetWindowContentRect(inWindowRef, windowRect);
	::CallBackErrorHandler(error);
	
	error = ::sn_ScrollbarsOnOff(inWindowRef, false);
	::CallBackErrorHandler(error);
	
	// to turn on orbits, you could save a settings file somewhere, then turn 
	// on orbits by using the settings file you have saved
	//SNCallBack long UseOtherSettings(inWindowRef, long invRefNum, long inParID, pStr255 inName)
 	double helioEclipticLong = 0.0;
 	double helioEclipticlat = 5.0*kDegreesToRadians;
 	double distAU = 0.05;
	error = ::sn_SetViewerLocation(inWindowRef, helioEclipticlat, helioEclipticLong, distAU, false); //move to the 0, 0 location, .1 
	::CallBackErrorHandler(error);
	
	error = ::sn_SetFieldOfView(inWindowRef, 90.0*kDegreesToRadians);
	::CallBackErrorHandler(error);
	
	// find, centre, and lock on the sun (don't zoom)
	PSelection sunSelection;
	error = ::sn_FindObject(inWindowRef, "\pSun", "\pPlanets", &sunSelection, true, false, true, true);
	::CallBackErrorHandler(error);

	
	// turn on some guides:
	error = ::sn_TurnGuidesOnOff(inWindowRef, kEclipticGuidesLayer, k3DAxes, true);
	::CallBackErrorHandler(error);
	
	// turn those pesky feet off...
	//---------------------------
	error = ::sn_TurnFeetOnOff(inWindowRef, false);
	::CallBackErrorHandler(error);
	
	for (long count = 1; count < 400; count++)
	{
		helioEclipticLong += 1.5*kDegreesToRadians;
		distAU = GetNextElevation(distAU);
		error = ::sn_SetViewerLocation(inWindowRef, helioEclipticlat, helioEclipticLong, distAU, false); //move to the 0, 0 location, .1 
		::CallBackErrorHandler(error);
		error = ::sn_ImmediateUpdateWindow(inWindowRef);
		::CallBackErrorHandler(error);
	}
}


//----------------------------------------------------------------------
//	HandleError
//
//	Purpose:	Called when an actual error occurs
//					
//	Inputs:		inSNErr	description of error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/18	Tom			3.0			New Callback API
//
//----------------------------------------------------------------------

void HandleError(long /*inSNErr*/)
{
	return;
}


//----------------------------------------------------------------------
//	CallBackErrorHandler
//
//	Purpose:	This call is an outline of how you might handle an erorr 
// 				coming back from one of the Sienna API calls
//				In this version, I just have a place to put a break point
//				in with the debugger, but you could do anything.
//					
//	Inputs:		inSNErr	description of possible error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/18	Tom			3.0			New Callback API
//
//----------------------------------------------------------------------

void CallBackErrorHandler(long inSNErr)
{
	if (inSNErr != snNoErr)
	{
		::HandleError(inSNErr);
	}
	else
	{
		// everything is fine
	}
}


//----------------------------------------------------------------------
//	DoCallBackAPIDemo
//
//	Purpose:	This call just goes through and calls lots of the Callback APIs,
//				in order to test them for Sienna, and so you can see how they 
//				might be used. There are a lot of possible calls, but you might
//				only call one or two of them. Good luck.
//				If you find that a call is not working as you thought it should,
//				send me an email. It is just as likely that the problem is Sienna's and
//				not yours.
//				--Tom
//					
//	Inputs:		outIndicator	description of indicator
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/18	Tom			3.0			New Callback API
//
//----------------------------------------------------------------------

void DoCallBackAPIDemo(void)
{
	snErr error;
	sn_WindowRef windowRef1;
	error = ::sn_NewWindow(&windowRef1);
	::CallBackErrorHandler(error);
	
	sn_WindowRef windowRef2;
	error = ::sn_NewWindow(&windowRef2);
	::CallBackErrorHandler(error);
	
	sn_WindowRef windowRef3;
	error = ::sn_NewWindow(&windowRef3);
	::CallBackErrorHandler(error);
	
	error = ::sn_ScrollbarsOnOff(windowRef3, false);
	::CallBackErrorHandler(error);
	
	// Lets find the moon and zoom in on it:
	PSelection moonSelection;
	error = ::sn_FindObject(windowRef3, "\pMoon", "\pPlanets", &moonSelection, true, true, true, true);
	::CallBackErrorHandler(error);
	
	// Lets find m42 and just get the info
	PSelection m42Selection;
	error = ::sn_FindObject(windowRef3, "\pM42", "\pMessier", &m42Selection, false, false, false, false);
	::CallBackErrorHandler(error);
	
	// grab a screen shot of window 3 (which should have the moon in it)
	//SaveScreenShotToDisk();
	pHandleToPicture pictureH = NULL;
	error = ::sn_GrabScreenshot(windowRef3, &pictureH);
	::CallBackErrorHandler(error);

	// the screen shot you get here can be converted into a GIF, jpeg or whatever, 
	// and done with it what you will.
	// Since we now own this picture handle, we have to destroy it
	//-----------------------------------------------------------
#if TARGET_OS_WIN32
	DeleteObject(pictureH);
#elif TARGET_OS_MAC
	DisposeHandle((Handle) pictureH);
#else
	huh?
#endif	
	pictureH = NULL;
	
	// now bring window 2 to the front:
	error = ::sn_BringDocumentWindowToFront(windowRef2);
	::CallBackErrorHandler(error);
	error = ::sn_ImmediateUpdateWindow(windowRef2);
	::CallBackErrorHandler(error);
	
	// now go to the alt az of 20deg alt, 50 deg azimuth
	double alt = 20.0*kDegreesToRadians;
	double az = 50.0*kDegreesToRadians;
	double mathAz;
	// need to convert azimuth to mathematical azimuth
	::sn_RealWorldAzimuth2Mathematical(az, &mathAz);
	PSphericalCoordinates altAz;
	altAz.ra = mathAz;
	altAz.dec = alt;
	altAz.radius = 1.0; // coordinates are at radius 1.
	PXYZCoordinates altAzXYZ;
	error = ::sn_SphericalToXYZ(altAz, &altAzXYZ);
	::CallBackErrorHandler(error);
	error = ::sn_CentreWindowAtCoords(windowRef2, kSNAltAzSystem, altAzXYZ, false, true, 30*kDegreesToRadians, false);
	::CallBackErrorHandler(error);
	error = ::sn_ImmediateUpdateWindow(windowRef2);
	::CallBackErrorHandler(error);
	
	// now lets move the viewer in window 1 over to the moon and lock on the earth
	error = ::sn_BringDocumentWindowToFront(windowRef1);
	::CallBackErrorHandler(error);
	
	error = ::sn_GotoPlanet(windowRef1, "\pMoon");
	::CallBackErrorHandler(error);
	
	error = ::sn_SetViewerLocation(windowRef1, 0.0, 0.0, 3.0*kMetersToAu, false); //move to the 0, 0 location, 3m up 
	::CallBackErrorHandler(error);
	
	error = ::sn_TurnHorizonOnOff(windowRef1, false, kHorizonOption_noChange);
	::CallBackErrorHandler(error);
	
	error = ::sn_ImmediateUpdateWindow(windowRef1);
	::CallBackErrorHandler(error);
	
	PSelection earthSelection;
	error = ::sn_FindObject(windowRef1, "\pEarth", "\pPlanets", &earthSelection, true, true, true, true);
	::CallBackErrorHandler(error);
	
	// now lets move the viewer in window 1 over to the moon and lock on the earth
	// lets go do a spiral trip out from the sun in a fixed location, at, say 5 deg up from the 
	// Sun
	::TakeWhirlWindTour(windowRef1);
	::CallBackErrorHandler(error);
	
	error = sn_CloseWindow(windowRef2);
	::CallBackErrorHandler(error);
}

//----------------------------------------------------------------------
//	DoDrawIndicator
//
//	Purpose:	Since we didn't pass back fLayer as Depth_None, we get called everytime
//				Starry Night's view is updated. We then pass back in PIndicator
//				a description of a field of view indicator we would like to have
//				drawn. If you don't want an indicator drawn at the current scope
//				location then pass back fLayer as Depth_None in DoInitialize.
//					
//	Inputs:		outIndicator	description of indicator
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoDrawIndicator(PIndicator* outIndicator)
{
	outIndicator->fDrawIndicator = true;					// true, we want an indicator drawn
	outIndicator->fIndicatorStyle = Indicator_Circle;	// type of indicator

	outIndicator->fDoneDrawing = true;					// this is our only indicator

	GetTelescopeRaDec(&outIndicator->fCentreRA, &outIndicator->fCentreDec);
	outIndicator->fVertical = GetTelescopeFieldOfView();
	outIndicator->fHorizontal = 0.0;					// not used for circular indicators
	outIndicator->fRollAngle = 0.0;						// not used for circular indicators

	PStringAssign(outIndicator->fString, "Our Telescope");
	
	// If you wish you can also set the colour of the indicator by setting
	// fRed, fGreen, fBlue.
}

//----------------------------------------------------------------------
//	DoBuildContextualPopup
//
//	Purpose:	If the user right-clicks on an object then Starry Night builds a
//				contextual popup menu. You can choose to add items to this menu if
//				you wish. Our example plug-in adds two items: Slew and Synch.
//					
//	Inputs:		inBuildContext	record describing object for which we are building a
//								menu, as well as information about the menu itself.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------
#if TARGET_OS_MAC
	long gSlewMenuItem = 0;
	long gSynchMenuItem = 0;
#endif

void DoBuildContextualPopup(const PBuildContext* inBuildContext)
{
	// In this example we don't really care about what object has been right clicked
	// so we don't look at inBuildContext->fObject.
	
	// on the Mac we are passed a MenuHandle
#if TARGET_OS_MAC
	// add the seperator
	::MacAppendMenu(inBuildContext->fMenuHandle, "\p-(");
	
	// The MacOS only tracks menu items. We therefore need to keep track of the menu item for each of the two commands
	::MacAppendMenu(inBuildContext->fMenuHandle, "\pHi"); // better to set text with SetMenuItemText
	gSlewMenuItem = ::CountMItems(inBuildContext->fMenuHandle);
	::SetMenuItemText(inBuildContext->fMenuHandle, gSlewMenuItem, "\pSlew Telescope");
	
	::MacAppendMenu(inBuildContext->fMenuHandle, "\pHi"); // better to set text with SetMenuItemText
	gSynchMenuItem = ::CountMItems(inBuildContext->fMenuHandle);
	::SetMenuItemText(inBuildContext->fMenuHandle, gSynchMenuItem, "\pSynch Telescope");
#endif
	
	// For the PC we are passed an HMENU, 
#if TARGET_OS_WIN32
	char	itemText[255];
	// Add divider line to divide our telescope controller items from the others
	unsigned long flags = MF_SEPARATOR;
	bool success = ::AppendMenu(inBuildContext->fMenuHandle, flags, 0, "-");

	// Add Slew item using a command of Command_Slew
	flags = 0;
	strcpy(itemText, "Slew Telescope");
	success = ::AppendMenu(inBuildContext->fMenuHandle, flags, Command_Slew, itemText);

	// Add Synch item using a command of Command_Synch
	strcpy(itemText, "Synch Telescope");
	success = ::AppendMenu(inBuildContext->fMenuHandle, flags, Command_Synch, itemText);
#endif
}

//----------------------------------------------------------------------
//	DoContextualPopup
//
//	Purpose:	We get called whenever the user selects an item from the contextual
//				popup menu. We check whether it is one of the items that we added and
//				handle it appropriately.
//					
//	Inputs:		inContext	record describing object for which we are building a
//							menu, as well as information about the menu item that
//							was selected.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

void DoContextualPopup(const PDoContext* inContext)
{
	long command = 0;
#if TARGET_OS_MAC
	if (inContext->fMenuItem == gSlewMenuItem)
		command = Command_Slew;
	else if (inContext->fMenuItem == gSynchMenuItem)
		command = Command_Synch;
#endif

#if TARGET_OS_WIN32
	command = inContext->fMenuCommand;
#endif

	// Look for one of the menu commands that we added
	if (command == Command_Slew)
	{
		SlewTelescope(inContext->fObject.fRightAscension, inContext->fObject.fDeclination);
	}
	else if (command == Command_Synch)
	{
		SynchTelescope(inContext->fObject.fRightAscension, inContext->fObject.fDeclination);
	}
}

//----------------------------------------------------------------------
//	DoOperation
//
//	Purpose:	Starry Night communicates with your plug-in through one call, DoOperation.
//				(and hence DoOperation is exported using EXP).
//					
//	Inputs:		inOperation	the operation that Starry Night wishes your plug-in to perform
//							(StarryNightPlugins.h describes each operation). You need not
//							support every operation. This example responds to all the
//							operations that a telescope controller plug-in will need to
//							respond to. You may chose to respond to others as well, if you
//							wish.
//
//	I/O:		ioParams	a parameter block that passes various information back and forth
//							between Starry Night and the plug-in. Each operation uses
//							a different paramater block or none at all, in which case
//							ioParams will be NULL. The description of the various operations
//							(in StarryNightPlugins.h) describes what ioParams means for each
//							one.
//
//	Outputs:	short		Unused. You should always pass back noError here.
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------

SNCall short DoOperation(OperationT inOperation, void* ioParams)
{
	// When there is no connection to the telescope, we don't respond to many operations
	switch (inOperation)
	{
		case op_Initialize:
			DoInitialize((PInitialize*) ioParams);
			break;
		case op_Terminate:
			// do any cleanup here. You receive op_Terminate when Starry Night quits.
			break;
		case op_MenuStatus:
			DoMenuStatus((PMenuStatus*) ioParams);
			break;
		case op_TelescopeConfigure:
			DoTelescopeConfigure();
			break;
		case op_TelescopeTime:
			if (gConnectionOpened)
				DoTelescopeTime((PTime*) ioParams);
			break;
		case op_TelescopeLocation:
			if (gConnectionOpened)
				DoTelescopeLocation((PLocation*) ioParams);
			break;
		case op_TelescopeOpen:
			DoTelescopeOpen();
			break;
		case op_TelescopeClose:
			DoTelescopeClose();
			break;
		case op_TelescopeTrack:
			if (gConnectionOpened)
				DoTelescopeTrack();
			break;
		case op_TelescopeSlew:
			if (gConnectionOpened)
				SlewTelescope(((PGaze*) ioParams)->fGazeRa, ((PGaze*) ioParams)->fGazeDec);
			break;
		case op_TelescopeSync:
			if (gConnectionOpened)
				SynchTelescope(((PGaze*) ioParams)->fGazeRa, ((PGaze*) ioParams)->fGazeDec);
			break;
		case op_Idle:
			if (gConnectionOpened)
				DoIdle((PEvent*) ioParams);
			break;
		case op_DrawIndicator:
			if (gConnectionOpened)
				DoDrawIndicator((PIndicator*) ioParams);
			break;
		case op_BuildContextualPopup:
			if (gConnectionOpened)
				DoBuildContextualPopup((PBuildContext*) ioParams);
			break;
		case op_DoContextualPopup:
			if (gConnectionOpened)
				DoContextualPopup((PDoContext*) ioParams);
			break;
	}
	return noError;
}
